//
// file :         acqthread.cpp
//
// description :  gige camera acquisition thread
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//

#include "acqthread.h"

#include <malloc.h>

//using namespace Pylon;

#ifndef ASSERT
#define	ASSERT(c)
#endif

void acqthread::insert_watermark(void *array, uint32_t value, ArrayType type) 
{
	int step = (type == SHORT_TYPE) ? 2 : 1;
	uint8_t *array8 = (uint8_t *)array;
	uint16_t *array16 = (uint16_t *)array;

	// Insert the 32-bit value into the first 32 elements (LSB only)
	for (int i = 0; i < 32; i++) {
		if (type == SHORT_TYPE)
			array16[i] = (array16[i] & 0xFFFE) | ((value >> i) & 0x1);
		else
			array8[i] = (array8[i] & 0xFE) | ((value >> i) & 0x1);
	}

	// Get timestamps
	struct timespec ts;
	clock_gettime(CLOCK_REALTIME, &ts);
	uint64_t realtime_seconds = (uint64_t)ts.tv_sec;
	uint64_t realtime_microseconds = (uint64_t)(ts.tv_nsec / 1000);

	struct timeval tv;
	gettimeofday(&tv, NULL);
	struct tm gmt;
	gmtime_r(&tv.tv_sec, &gmt);
	uint64_t gmt_seconds = (uint64_t)mktime(&gmt);
	uint64_t gmt_microseconds = (uint64_t)tv.tv_usec;

	// Insert timestamps
	for (int i = 0; i < 64; i++) {
		if (type == SHORT_TYPE) {
			array16[32 + i] = (array16[32 + i] & 0xFFFE) | ((realtime_seconds >> i) & 0x1);
			array16[32 + 64 + i] = (array16[32 + 64 + i] & 0xFFFE) | ((realtime_microseconds >> i) & 0x1);
			array16[32 + 128 + i] = (array16[32 + 128 + i] & 0xFFFE) | ((gmt_seconds >> i) & 0x1);
			array16[32 + 192 + i] = (array16[32 + 192 + i] & 0xFFFE) | ((gmt_microseconds >> i) & 0x1);
		} 
		else {
			array8[32 + i] = (array8[32 + i] & 0xFE) | ((realtime_seconds >> i) & 0x1);
			array8[32 + 64 + i] = (array8[32 + 64 + i] & 0xFE) | ((realtime_microseconds >> i) & 0x1);
			array8[32 + 128 + i] = (array8[32 + 128 + i] & 0xFE) | ((gmt_seconds >> i) & 0x1);
			array8[32 + 192 + i] = (array8[32 + 192 + i] & 0xFE) | ((gmt_microseconds >> i) & 0x1);
		}
	}
}


/*
 * acqthread::update_configuration()
 * Update data structures on the fly
 */
void acqthread::update_configuration(void)
{

	DEBUG_STREAM << __FUNCTION__ << ", start update_configuration()" << endl;

	/* if this is the first run */
	if (myacq->cam_param[myacq->cam_idx].configured == false) {

		INFO_STREAM << __FUNCTION__ << ", default parameters camera initialization" << endl;

		/* overload manual roi param in case max image size doesn't match */
		if ((myacq->cam_param[myacq->cam_idx].manual_roi_param[0] > width) ||
				(myacq->cam_param[myacq->cam_idx].manual_roi_param[2] > width) ||
				(myacq->cam_param[myacq->cam_idx].manual_roi_param[1] > height) ||
				(myacq->cam_param[myacq->cam_idx].manual_roi_param[3] > height) ||
				(myacq->cam_param[myacq->cam_idx].manual_roi_param[0] < 0) ||
				(myacq->cam_param[myacq->cam_idx].manual_roi_param[1] < 0) ||
				(myacq->cam_param[myacq->cam_idx].manual_roi_param[2] < 0) ||
				(myacq->cam_param[myacq->cam_idx].manual_roi_param[3] < 0)) {
			myacq->cam_param[myacq->cam_idx].manual_roi_param[0] = 0;
			myacq->cam_param[myacq->cam_idx].manual_roi_param[1] = 0;
			myacq->cam_param[myacq->cam_idx].manual_roi_param[2] = width;
			myacq->cam_param[myacq->cam_idx].manual_roi_param[3] = height;
		}

		/* get sensor width and height */
		myacq->cam_param[myacq->cam_idx].max_width = get_sensor_width();    // XXXYYY
		myacq->cam_param[myacq->cam_idx].max_height = get_sensor_width();



	}

	if (myacq->cam_param[myacq->cam_idx].img_depth == 8) {

		DEBUG_STREAM << __FUNCTION__ << ", initialize ImageProc 8-bit object" << endl;

		/* set horizontal/vertical reference position offset */
		if (myacq->take_reference == true) {
			myacq->take_reference = false;
			for (int32_t i = 0; i < myacq->imageBufferSize; i++) {
				myacq->img8[i]->ref_hor_pos_offset = myacq->img8[myacq->img_cur]->hor_pos + 2 * myacq->img8[myacq->img_cur]->hor_pos_offset;
				myacq->img8[i]->ref_ver_pos_offset = myacq->img8[myacq->img_cur]->ver_pos;					
			}
		}
		/* set image size */
		myacq->img8[myacq->img_nxt]->set_size(get_width(), get_height());

		/* set low pass filter order */
		myacq->img8[myacq->img_nxt]->set_lp_order(myacq->cam_param[myacq->cam_idx].lp_order);
		/* set decimation factor */
		myacq->img8[myacq->img_nxt]->set_hor_grid(hor_grid_size);
		myacq->img8[myacq->img_nxt]->set_ver_grid(ver_grid_size);

		/* set mirror */
		myacq->img8[myacq->img_nxt]->set_mirror(myacq->cam_param[myacq->cam_idx].mirror);
		/* enable image processing */
		myacq->img8[myacq->img_nxt]->set_enable_process(myacq->cam_param[myacq->cam_idx].process_enable);
		/* set auto roi */
		myacq->img8[myacq->img_nxt]->set_enable_auto_roi(myacq->cam_param[myacq->cam_idx].auto_roi);
		/* set roi threshold */
		myacq->img8[myacq->img_nxt]->set_roi_threshold(myacq->cam_param[myacq->cam_idx].roi_threshold);
		myacq->img8[myacq->img_nxt]->set_search_background_param(myacq->cam_param[myacq->cam_idx].search_background_param);
		/* plot mode and level */
		myacq->img8[myacq->img_nxt]->set_plot_mode(myacq->cam_param[myacq->cam_idx].plot_mode);
		myacq->img8[myacq->img_nxt]->set_plot_level(myacq->cam_param[myacq->cam_idx].plot_level);
		myacq->img8[myacq->img_nxt]->set_plot_offset_axis(myacq->cam_param[myacq->cam_idx].plot_offset_axis);		
		/* enable fast calculation */
		myacq->img8[myacq->img_nxt]->set_enable_fast_calc(myacq->cam_param[myacq->cam_idx].fast_calc);
		/* enable median filtering */
		myacq->img8[myacq->img_nxt]->set_enable_median_filter(myacq->cam_param[myacq->cam_idx].enable_median_filter);
		/* set manual roi */
		myacq->img8[myacq->img_nxt]->set_enable_manual_roi(myacq->cam_param[myacq->cam_idx].sw_roi);
		myacq->img8[myacq->img_nxt]->set_manual_roi(myacq->cam_param[myacq->cam_idx].manual_roi_param);
		/* set process mode */
		myacq->img8[myacq->img_nxt]->set_algorithm(myacq->cam_param[myacq->cam_idx].process_mode);
		/* update fitting timeout */
		if (myacq->frequency > 0)
			myacq->img8[myacq->img_nxt]->set_fit_tout(1000/myacq->frequency/2); /* leave half of the time period for fitting */
		/* digital image stabilization parameters */
		myacq->img8[myacq->img_nxt]->set_ref_raw_pos(myacq->cam_param[myacq->cam_idx].ref_raw_hor_pos, myacq->cam_param[myacq->cam_idx].ref_raw_ver_pos);
		myacq->img8[myacq->img_nxt]->set_enable_dis(myacq->cam_param[myacq->cam_idx].dis_enable);


		/* if rotation not multiple of 90, enable rotation interpolation */
		if ((int32_t)myacq->cam_param[myacq->cam_idx].rotation % 90) {
			myacq->img8[myacq->img_nxt]->set_enable_rotation_filter(true);
		}
		else
			myacq->img8[myacq->img_nxt]->set_enable_rotation_filter(false);

		DEBUG_STREAM << __FUNCTION__ << ", ImageProc 8-bit object initialized" << endl;

	}
	else if (myacq->cam_param[myacq->cam_idx].img_depth == 16) {

		DEBUG_STREAM << __FUNCTION__ << ", initialize ImageProc 16-bit object" << endl;

		/* set horizontal/vertical reference position offset */
		if (myacq->take_reference == true) {
			myacq->take_reference = false;
			for (int32_t i = 0; i < myacq->imageBufferSize; i++) {
				myacq->img16[i]->ref_hor_pos_offset = myacq->img16[myacq->img_cur]->hor_pos  + 2 * myacq->img16[myacq->img_cur]->hor_pos_offset;
				myacq->img16[i]->ref_ver_pos_offset = myacq->img16[myacq->img_cur]->ver_pos;	
			}
		}

		/* set image size */
		myacq->img16[myacq->img_nxt]->set_size((int32_t)get_width(), (int32_t) get_height());

		/* set low pass filter order */
		myacq->img16[myacq->img_nxt]->set_lp_order(myacq->cam_param[myacq->cam_idx].lp_order);
		/* set decimation factor */
		myacq->img16[myacq->img_nxt]->set_hor_grid(hor_grid_size);
		myacq->img16[myacq->img_nxt]->set_ver_grid(ver_grid_size);

		/* set mirror */
		myacq->img16[myacq->img_nxt]->set_mirror(myacq->cam_param[myacq->cam_idx].mirror);
		/* enable image processing */
		myacq->img16[myacq->img_nxt]->set_enable_process(myacq->cam_param[myacq->cam_idx].process_enable);
		/* set auto roi */
		myacq->img16[myacq->img_nxt]->set_enable_auto_roi(myacq->cam_param[myacq->cam_idx].auto_roi);
		/* set roi threshold */
		myacq->img16[myacq->img_nxt]->set_roi_threshold(myacq->cam_param[myacq->cam_idx].roi_threshold);
		myacq->img16[myacq->img_nxt]->set_search_background_param(myacq->cam_param[myacq->cam_idx].search_background_param);
		/* plot mode and level */
		myacq->img16[myacq->img_nxt]->set_plot_mode(myacq->cam_param[myacq->cam_idx].plot_mode);
		myacq->img16[myacq->img_nxt]->set_plot_level(myacq->cam_param[myacq->cam_idx].plot_level);
		myacq->img16[myacq->img_nxt]->set_plot_offset_axis(myacq->cam_param[myacq->cam_idx].plot_offset_axis);
		/* enable fast calculation */
		myacq->img16[myacq->img_nxt]->set_enable_fast_calc(myacq->cam_param[myacq->cam_idx].fast_calc);
		/* enable median filtering */
		myacq->img16[myacq->img_nxt]->set_enable_median_filter(myacq->cam_param[myacq->cam_idx].enable_median_filter);
		/* set manual roi */
		myacq->img16[myacq->img_nxt]->set_enable_manual_roi(myacq->cam_param[myacq->cam_idx].sw_roi);
		myacq->img16[myacq->img_nxt]->set_manual_roi(myacq->cam_param[myacq->cam_idx].manual_roi_param);
		/* set process mode */
		myacq->img16[myacq->img_nxt]->set_algorithm(myacq->cam_param[myacq->cam_idx].process_mode);

		/* update fitting timeout */
		if (myacq->frequency > 0)
			myacq->img16[myacq->img_nxt]->set_fit_tout(1000/myacq->frequency/2); /* leave half of the time period for fitting */

		/* digital image stabilization parameters */
		myacq->img16[myacq->img_nxt]->set_ref_raw_pos(myacq->cam_param[myacq->cam_idx].ref_raw_hor_pos, myacq->cam_param[myacq->cam_idx].ref_raw_ver_pos);
		myacq->img16[myacq->img_nxt]->set_enable_dis(myacq->cam_param[myacq->cam_idx].dis_enable);

		/* if rotation not multiple of 90, enable rotation interpolation */
		if ((int32_t)myacq->cam_param[myacq->cam_idx].rotation % 90)
			myacq->img16[myacq->img_nxt]->set_enable_rotation_filter(true);
		else
			myacq->img16[myacq->img_nxt]->set_enable_rotation_filter(false);

		DEBUG_STREAM << __FUNCTION__ << ", ImageProc 16-bit object initialized" << endl;

	}

	myacq->cam_param[myacq->cam_idx].configured = true;

	DEBUG_STREAM << __FUNCTION__ << ", end update_configuration()" << endl;

}

/*
 * acqthread::save_data()
 * Save data into circular buffers
*/
bool acqthread::save_data(void *buf, uint32_t curr_frame)
{
	struct timeval tv;
	bool avg_thres_ok = true;

	gettimeofday(&tv,NULL);

	/* 8 bit */
	if (myacq->cam_param[myacq->cam_idx].img_depth == 8) {

		DEBUG_STREAM << __FUNCTION__ << ", saving 8-bit image" << endl;

		/* rotate image */
		if (myacq->trigger_background) {
			myacq->cam_param[myacq->cam_idx].bckimg8->push_permutation((unsigned char*) buf,
				myacq->cam_param[myacq->cam_idx].img_permutation,
				curr_frame , (uint32_t)  myacq->bunch_number);
			myacq->trigger_background = false;
		}
		if (myacq->cam_param[myacq->cam_idx].background_subtraction) {
			myacq->img8[myacq->img_nxt]->push_permutation_with_background((unsigned char*) buf, myacq->cam_param[myacq->cam_idx].img_permutation,
				curr_frame , (uint32_t) myacq->bunch_number,(const unsigned char*)myacq->cam_param[myacq->cam_idx].bckimg8->image);
		}
		else {
			myacq->img8[myacq->img_nxt]->push_permutation((unsigned char*) buf, myacq->cam_param[myacq->cam_idx].img_permutation, 
				curr_frame , (uint32_t)myacq->bunch_number);
		}

		insert_watermark(myacq->img8[myacq->img_nxt]->image, (uint32_t) myacq->bunch_number,CHAR_TYPE);

	}
	/* 16 bit */
	else if (myacq->cam_param[myacq->cam_idx].img_depth == 16) {

		DEBUG_STREAM << __FUNCTION__ << ", saving 16-bit image" << endl;
	
		if (myacq->trigger_background) {
			myacq->cam_param[myacq->cam_idx].bckimg16->push_permutation((unsigned short*)buf, 
				myacq->cam_param[myacq->cam_idx].img_permutation,
				curr_frame ,(uint32_t) myacq->bunch_number);
			myacq->trigger_background = false;
		}
		if (myacq->cam_param[myacq->cam_idx].background_subtraction) {
			myacq->img16[myacq->img_nxt]->push_permutation_with_background((unsigned short*) buf, myacq->cam_param[myacq->cam_idx].img_permutation,
				curr_frame , (uint32_t) myacq->bunch_number,(const unsigned short *)myacq->cam_param[myacq->cam_idx].bckimg16->image);
		}
		else {
			myacq->img16[myacq->img_nxt]->push_permutation((unsigned short*) buf, myacq->cam_param[myacq->cam_idx].img_permutation, 
				curr_frame , (uint32_t) myacq->bunch_number);
		}

		insert_watermark(myacq->img16[myacq->img_nxt]->image, (uint32_t) myacq->bunch_number,SHORT_TYPE);

		unsigned short *array = (unsigned short *)myacq->img16[myacq->img_nxt]->image;

	}

	myacq->img_cur = myacq->img_nxt;

	myacq->img_nxt++;

	/* save data into extended circular buffers */
	if (myacq->cam_param[myacq->cam_idx].img_depth == 8) {

		DEBUG_STREAM << __FUNCTION__ << ", saving 8-bit circular buffers" << endl;

		if (*myacq->attr_ImageSumSamples_read) {
			myacq->in_mutex->lock();
			myacq->imagesum_counter++;
			int img_sub_idx;
			if (myacq->imagesum_counter > *myacq->attr_ImageSumSamples_read) {
				img_sub_idx = (myacq->img_cur + myacq->imageBufferSize - *myacq->attr_ImageSumSamples_read) % myacq->imageBufferSize;
				for (int i = 0; i < myacq->img8[myacq->img_cur]->get_width()*myacq->img8[myacq->img_cur]->get_height(); i++) {
					myacq->attr_ImageSum_read[i] += (Tango::DevLong) myacq->img8[myacq->img_cur]->image[i]-(Tango::DevLong) myacq->img8[img_sub_idx]->image[i];
				}
			}
			else {
				for (int i = 0; i < myacq->img8[myacq->img_cur]->get_width()*myacq->img8[myacq->img_cur]->get_height(); i++) {
					myacq->attr_ImageSum_read[i] += (Tango::DevLong) myacq->img8[myacq->img_cur]->image[i];
				}
			}
			myacq->in_mutex->unlock();
		}

		if (myacq->circ_buffer) {
			push_data((void *) &myacq->img8[myacq->img_cur]->phase, tv, myacq->bunch_number, myacq->buf_phase);
			push_data((void *) &myacq->img8[myacq->img_cur]->area, tv, myacq->bunch_number, myacq->buf_area);
			push_data((void *) &myacq->img8[myacq->img_cur]->hor_pos, tv, myacq->bunch_number, myacq->buf_hor_pos);
			push_data((void *) &myacq->img8[myacq->img_cur]->hor_sigma, tv, myacq->bunch_number, myacq->buf_hor_sigma);
			push_data((void *) &myacq->img8[myacq->img_cur]->saturation, tv, myacq->bunch_number, myacq->buf_saturation);
			push_data((void *) &myacq->img8[myacq->img_cur]->ver_pos, tv, myacq->bunch_number, myacq->buf_ver_pos);
			push_data((void *) &myacq->img8[myacq->img_cur]->ver_sigma, tv, myacq->bunch_number, myacq->buf_ver_sigma);
			push_data((void *) &myacq->img8[myacq->img_cur]->intensity, tv, myacq->bunch_number, myacq->buf_intensity);
			push_data((void *) &myacq->img8[myacq->img_cur]->fit_error, tv, myacq->bunch_number, myacq->buf_fit_error);
			push_data((void *) &curr_frame, tv, myacq->bunch_number, myacq->buf_acquisition_counter);
			push_data((void *) &myacq->img8[myacq->img_cur]->max_value, tv, myacq->bunch_number, myacq->buf_max_val);
			push_data((void *) myacq->attr_Gain_read, tv, myacq->bunch_number, myacq->buf_gain);
			push_data((void *) myacq->attr_Exposure_read, tv, myacq->bunch_number, myacq->buf_exposure);

			double mean_intensity = 0;
			if ((myacq->img8[myacq->img_cur]->area != 0) && (myacq->img8[myacq->img_cur]->hor_calibration != 0) &&
				(myacq->img8[myacq->img_cur]->ver_calibration != 0)) {
				mean_intensity = myacq->img8[myacq->img_cur]->intensity / 
													myacq->img8[myacq->img_cur]->area /
													myacq->img8[myacq->img_cur]->hor_calibration /
													myacq->img8[myacq->img_cur]->ver_calibration;
			}
			double hor_pos = myacq->img8[myacq->img_cur]->hor_pos, ver_pos = myacq->img8[myacq->img_cur]->ver_pos;
			/* force rnm value to 0 in case of fitting error or signal under threshold */
			*myacq->attr_MeanIntensity_read = mean_intensity;
			if ((mean_intensity < *myacq->attr_MeanIntensityThreshold_read) || myacq->img8[myacq->img_cur]->fit_error) {
				hor_pos = NAN; ver_pos = NAN;
				if (*myacq->attr_FeedbackMode_read) {
					if (*myacq->attr_MeanIntensity_read < (double)*myacq->attr_MeanIntensityThreshold_read) {
						avg_thres_ok = false;
						myacq->set_status("signal under threshold");
					}
					else if (myacq->img8[myacq->img_cur]->fit_error) {
						avg_thres_ok = false;
						myacq->set_status("profile fitting failed");
					}	
				}
			}

		}

		DEBUG_STREAM << __FUNCTION__ << ", 8-bit circular buffers saved" << endl;

	}
	else if (myacq->cam_param[myacq->cam_idx].img_depth == 16) {

		DEBUG_STREAM << __FUNCTION__ << ", saving 16-bit circular buffers" << endl;

		if (*myacq->attr_ImageSumSamples_read) {
			myacq->in_mutex->lock();
			myacq->imagesum_counter++;
			int img_sub_idx;
			if (myacq->imagesum_counter >= *myacq->attr_ImageSumSamples_read) {
				img_sub_idx = (myacq->img_cur + myacq->imageBufferSize - *myacq->attr_ImageSumSamples_read) % myacq->imageBufferSize;
				for (int i = 0; i < myacq->img16[myacq->img_cur]->get_width()*myacq->img16[myacq->img_cur]->get_height(); i++) {
					myacq->attr_ImageSum_read[i] += (Tango::DevLong) myacq->img16[myacq->img_cur]->image[i]-(Tango::DevLong) myacq->img16[img_sub_idx]->image[i];
				}
			}
			else {
				for (int i = 0; i < myacq->img16[myacq->img_cur]->get_width()*myacq->img16[myacq->img_cur]->get_height(); i++) {
					myacq->attr_ImageSum_read[i] += (Tango::DevLong) myacq->img16[myacq->img_cur]->image[i];
				}
			}
			myacq->in_mutex->unlock();
		}

		if (myacq->circ_buffer) {
			push_data((void *) &myacq->img16[myacq->img_cur]->phase, tv, myacq->bunch_number, myacq->buf_phase);
			push_data((void *) &myacq->img16[myacq->img_cur]->area, tv, myacq->bunch_number, myacq->buf_area);
			push_data((void *) &myacq->img16[myacq->img_cur]->hor_pos, tv, myacq->bunch_number, myacq->buf_hor_pos);
			push_data((void *) &myacq->img16[myacq->img_cur]->hor_sigma, tv, myacq->bunch_number, myacq->buf_hor_sigma);
			push_data((void *) &myacq->img16[myacq->img_cur]->saturation, tv, myacq->bunch_number, myacq->buf_saturation);
			push_data((void *) &myacq->img16[myacq->img_cur]->ver_pos, tv, myacq->bunch_number, myacq->buf_ver_pos);
			push_data((void *) &myacq->img16[myacq->img_cur]->ver_sigma, tv, myacq->bunch_number, myacq->buf_ver_sigma);
			push_data((void *) &myacq->img16[myacq->img_cur]->intensity, tv, myacq->bunch_number, myacq->buf_intensity);
			push_data((void *) &myacq->img16[myacq->img_cur]->fit_error, tv, myacq->bunch_number, myacq->buf_fit_error);
			push_data((void *) &curr_frame, tv, myacq->bunch_number, myacq->buf_acquisition_counter);
			push_data((void *) &myacq->img16[myacq->img_cur]->max_value, tv, myacq->bunch_number, myacq->buf_max_val);
			push_data((void *) myacq->attr_Gain_read, tv, myacq->bunch_number, myacq->buf_gain);
			push_data((void *) myacq->attr_Exposure_read, tv, myacq->bunch_number, myacq->buf_exposure);
		}

		double mean_intensity = 0;
		if ((myacq->img16[myacq->img_cur]->area != 0) && (myacq->img16[myacq->img_cur]->hor_calibration != 0) &&
			(myacq->img16[myacq->img_cur]->ver_calibration != 0)) {
			mean_intensity = myacq->img16[myacq->img_cur]->intensity / 
												myacq->img16[myacq->img_cur]->area /
												myacq->img16[myacq->img_cur]->hor_calibration /
												myacq->img16[myacq->img_cur]->ver_calibration;
		}

		double hor_pos = myacq->img16[myacq->img_cur]->hor_pos,
						ver_pos = myacq->img16[myacq->img_cur]->ver_pos;
		/* force rnm value to 0 in case of fitting error or signal under threshold */
		*myacq->attr_MeanIntensity_read = mean_intensity; 
		if ((mean_intensity < *myacq->attr_MeanIntensityThreshold_read) || myacq->img16[myacq->img_cur]->fit_error) {
			hor_pos = NAN; ver_pos = NAN;
			if (*myacq->attr_FeedbackMode_read) {
				if (*myacq->attr_MeanIntensity_read < (double)*myacq->attr_MeanIntensityThreshold_read) {
					avg_thres_ok = false;
					myacq->set_status("signal under threshold");
				}
				else if (myacq->img8[myacq->img_cur]->fit_error) {
					avg_thres_ok = false;
					myacq->set_status("profile fitting failed");
				}	
			}
		}

		DEBUG_STREAM << __FUNCTION__ << ", 16-bit circular buffers saved" << endl;

	}

	if (myacq->img_nxt >= myacq->imageBufferSize)
		myacq->img_nxt = 0;

	if (avg_thres_ok) {
		*myacq->attr_FeedbackError_read = false;
	}
	else {
		*myacq->attr_FeedbackError_read = true;
	}

	return avg_thres_ok;

}


/*
 * TUCSENERR acqthread::init_data()
 * Initialize data structures of ImageProc class
 */
TUCSENERR acqthread::init_data(void)
{
	myacq->in_mutex->lock();

	TUCSENERR err = TUCSENERR_SUCCESS;

	{ string msg_log("init_data: start"); myacq->info_log(msg_log); }

	INFO_STREAM << __FUNCTION__ << ", start data initialization (ROI)" << endl;

	try {

		INFO_STREAM << __FUNCTION__ << ", initialize grid sizes" << endl;

		int64_t img_width;
		int64_t img_height;
		if (myacq->cam_param[myacq->cam_idx].hw_roi == false) {
			img_width = get_sensor_width()/myacq->cam_param[myacq->cam_idx].binning;   
			img_height = get_sensor_height()/myacq->cam_param[myacq->cam_idx].binning;
		}
		else {
			img_width = myacq->cam_param[myacq->cam_idx].hw_roi_param[2]-myacq->cam_param[myacq->cam_idx].hw_roi_param[0]+1;
			img_height = myacq->cam_param[myacq->cam_idx].hw_roi_param[3]-myacq->cam_param[myacq->cam_idx].hw_roi_param[1]+1;
		}

		/* vector used for coefficient permutation for image rotation */
		myacq->cam_param[myacq->cam_idx].img_permutation = (int32_t *) malloc(sizeof(int32_t)*img_height*img_width);
		{ string msg_log("init_data: malloc img_permutation done"); myacq->info_log(msg_log); }
		for (int32_t i = 0; i < (img_height*img_width); i++) {
			myacq->cam_param[myacq->cam_idx].img_permutation[i] = i;
		}

		/* initialize feedback counters and flags */
		myacq->cam_param[myacq->cam_idx].exposure_auto_cnt = 0;
		myacq->cam_param[myacq->cam_idx].gain_auto_cnt = 0;
		myacq->cam_param[myacq->cam_idx].gain_auto_once = false;
		myacq->cam_param[myacq->cam_idx].exposure_auto_once = false;

		/* optimize grid sizes */
		if ((myacq->cam_param[myacq->cam_idx].gridsize != 0) && (img_width != 0) &&
			(img_width >= myacq->cam_param[myacq->cam_idx].gridsize)) {
			hor_grid_size = (int)((double)img_width/(int)(img_width/myacq->cam_param[myacq->cam_idx].gridsize));
		}
		else {
			hor_grid_size = myacq->cam_param[myacq->cam_idx].gridsize;
		}
		/* set vertical grid */
		if ((myacq->cam_param[myacq->cam_idx].gridsize != 0) && (img_height != 0) &&
			(img_height >= myacq->cam_param[myacq->cam_idx].gridsize)) {
			ver_grid_size = (int)((double)img_height/(int)(img_width/myacq->cam_param[myacq->cam_idx].gridsize));
		}
		else {
			ver_grid_size = myacq->cam_param[myacq->cam_idx].gridsize;
		}

		{ string msg_log("init_data: optimizing grid sizes done"); myacq->info_log(msg_log); }

		memset(myacq->attr_ImageSum_read, 0, img_height * img_width * sizeof(Tango::DevLong));
		myacq->imagesum_counter = 0;

		

		if (myacq->cam_param[myacq->cam_idx].img_depth == 8) {

			INFO_STREAM << __FUNCTION__ << ", initialize 8-bit image ring buffer" << endl;

			/* initialize the image structure */
			for (int32_t i = 0; i < myacq->imageBufferSize; i++) {

				ImageProc <unsigned char> *tmp = new ImageProc <unsigned char>(img_height,img_width,(int32_t)myacq->fitNumCores);
				if (!tmp)
					return TUCSENERR_ERROR;

				tmp->set_lp_order((int32_t) myacq->cam_param[myacq->cam_idx].lp_order);
				tmp->set_hor_grid((int32_t) hor_grid_size);
				tmp->set_ver_grid((int32_t) ver_grid_size);
				tmp->binning = myacq->cam_param[myacq->cam_idx].binning;
				tmp->set_enable_auto_roi(myacq->autoRoi);
				tmp->set_enable_process(myacq->cam_param[myacq->cam_idx].process_enable);
				tmp->set_enable_median_filter(myacq->cam_param[myacq->cam_idx].enable_median_filter);
				tmp->set_mirror(myacq->cam_param[myacq->cam_idx].mirror);
				
				tmp-> set_enable_fast_calc(myacq->fastCalculation);
				tmp->set_plot_mode(myacq->plotMode[0]);
				tmp->set_roi_threshold(myacq->autoRoiParam[2]);
				myacq->img8.push_back(tmp);

				/* set calibration and mechanical offsets */
				tmp->hor_calibration = myacq->cam_param[myacq->cam_idx].hor_calibration;
				tmp->ver_calibration = myacq->cam_param[myacq->cam_idx].ver_calibration;
				tmp->hor_pos_offset = myacq->cam_param[myacq->cam_idx].hor_pos_offset * myacq->cam_param[myacq->cam_idx].hor_sign;
				//tmp->ver_pos_offset = myacq->cam_param[myacq->cam_idx].ver_pos_offset * myacq->cam_param[myacq->cam_idx].ver_sign;
				tmp->ver_pos_offset = myacq->cam_param[myacq->cam_idx].ver_pos_offset;

				tmp->hor_sign = myacq->cam_param[myacq->cam_idx].hor_sign;
				tmp->ver_sign = myacq->cam_param[myacq->cam_idx].ver_sign;

				tmp->ref_hor_pos_offset = myacq->cam_param[myacq->cam_idx].ref_hor_pos_offset + myacq->cam_param[myacq->cam_idx].delta_ref_hor_pos_offset;
				tmp->ref_ver_pos_offset = myacq->cam_param[myacq->cam_idx].ref_ver_pos_offset + myacq->cam_param[myacq->cam_idx].delta_ref_ver_pos_offset;

				/* hw roi change image offset */
				if (myacq->cam_param[myacq->cam_idx].hw_roi == true) {
					if (myacq->cam_param[myacq->cam_idx].mirror == false) { 
						tmp->img_hor_pos_offset = tmp->img_hor_pos_offset + (get_sensor_width()/2-      
							tmp->img_hor_pos_offset - myacq->cam_param[myacq->cam_idx].hw_roi_param[0]);
					}
					else {
						tmp->img_hor_pos_offset = tmp->img_hor_pos_offset + (get_sensor_width()/2-
							tmp->img_hor_pos_offset - myacq->cam_param[myacq->cam_idx].hw_roi_param[0]);
					}
					tmp->img_ver_pos_offset = tmp->img_ver_pos_offset + (get_sensor_height()/2-
						tmp->img_ver_pos_offset-myacq->cam_param[myacq->cam_idx].hw_roi_param[1]);
				}
				/* binning change image offset */
				if (myacq->cam_param[myacq->cam_idx].binning > 1) {
					if (myacq->cam_param[myacq->cam_idx].mirror == false) {
						tmp->img_hor_pos_offset = tmp->img_hor_pos_offset + (get_sensor_width()/myacq->cam_param[myacq->cam_idx].binning-
							tmp->img_hor_pos_offset - get_sensor_width()/2/myacq->cam_param[myacq->cam_idx].binning);
					}
					else {
						tmp->img_hor_pos_offset = tmp->img_hor_pos_offset + (get_sensor_width()/myacq->cam_param[myacq->cam_idx].binning-
							tmp->img_hor_pos_offset - get_sensor_width()/2/myacq->cam_param[myacq->cam_idx].binning);
					}
					tmp->img_ver_pos_offset = tmp->img_ver_pos_offset + (get_sensor_width()/myacq->cam_param[myacq->cam_idx].binning-
						tmp->img_ver_pos_offset-get_sensor_height()/2/myacq->cam_param[myacq->cam_idx].binning);
				}

			}

			/* initialize permutation vector */
			if (myacq->cam_param[myacq->cam_idx].mirror)
				myacq->img8[0]->permutation_mirrorh(myacq->cam_param[myacq->cam_idx].img_permutation);
			if (myacq->cam_param[myacq->cam_idx].rotation != 0)
				myacq->img8[0]->permutation_rotation(myacq->cam_param[myacq->cam_idx].rotation, myacq->cam_param[myacq->cam_idx].img_permutation);

			/* background subtraction support variable */
			myacq->cam_param[myacq->cam_idx].bckimg8 = 
				new ImageProc <unsigned char> (img_height,img_width,(int32_t)myacq->fitNumCores);

			myacq->cam_param[myacq->cam_idx].bckimg8->reset();
	
			INFO_STREAM << __FUNCTION__ << ", 8-bit image ring buffer initialized" << endl;

		} /* end image 8bit */

		else if (myacq->cam_param[myacq->cam_idx].img_depth == 16) {

			INFO_STREAM << __FUNCTION__ << ", initialize 16-bit image ring buffer" << endl;

			/* initialize the image structure */
			for (int32_t i = 0; i < myacq->imageBufferSize; i++) {

				ImageProc <unsigned short> *tmp = new ImageProc <unsigned short>(img_height,img_width,(int32_t)myacq->fitNumCores);
				if (!tmp)
					return TUCSENERR_ERROR;

				tmp->set_lp_order((int32_t)myacq->autoRoiParam[1]);
				tmp->set_hor_grid(hor_grid_size);
				tmp->set_ver_grid(ver_grid_size);
				tmp->binning = myacq->cam_param[myacq->cam_idx].binning;
				tmp->set_enable_auto_roi(myacq->autoRoi);
				tmp->set_enable_process(myacq->cam_param[myacq->cam_idx].process_enable);
				tmp->set_enable_median_filter(myacq->cam_param[myacq->cam_idx].enable_median_filter);
				tmp->set_mirror(myacq->cam_param[myacq->cam_idx].mirror);

				tmp-> set_enable_fast_calc(myacq->fastCalculation);
				tmp->set_plot_mode(myacq->plotMode[0]);
				tmp->set_roi_threshold(myacq->autoRoiParam[2]);
				myacq->img16.push_back(tmp);
				/* set calibration and mechanical offsets */
				tmp->hor_calibration = myacq->cam_param[myacq->cam_idx].hor_calibration;
				tmp->ver_calibration = myacq->cam_param[myacq->cam_idx].ver_calibration;
				tmp->hor_pos_offset = myacq->cam_param[myacq->cam_idx].hor_pos_offset * myacq->cam_param[myacq->cam_idx].hor_sign;
				tmp->ver_pos_offset = myacq->cam_param[myacq->cam_idx].ver_pos_offset * myacq->cam_param[myacq->cam_idx].ver_sign;
				tmp->hor_sign = myacq->cam_param[myacq->cam_idx].hor_sign;
				tmp->ver_sign = myacq->cam_param[myacq->cam_idx].ver_sign;				

				tmp->ref_hor_pos_offset = myacq->cam_param[myacq->cam_idx].ref_hor_pos_offset + myacq->cam_param[myacq->cam_idx].delta_ref_hor_pos_offset;
				tmp->ref_ver_pos_offset = myacq->cam_param[myacq->cam_idx].ref_ver_pos_offset + myacq->cam_param[myacq->cam_idx].delta_ref_ver_pos_offset;


				/* hw roi changes image offset */
				if (myacq->cam_param[myacq->cam_idx].hw_roi == true) {
					if (myacq->cam_param[myacq->cam_idx].mirror == false) {
						tmp->img_hor_pos_offset = tmp->img_hor_pos_offset + (get_sensor_width()/2-
							tmp->img_hor_pos_offset - myacq->cam_param[myacq->cam_idx].hw_roi_param[0]);
					}
					else {
						tmp->img_hor_pos_offset = tmp->img_hor_pos_offset + (get_sensor_width()/2-
							tmp->img_hor_pos_offset - myacq->cam_param[myacq->cam_idx].hw_roi_param[0]);
					}
					tmp->img_ver_pos_offset = tmp->img_ver_pos_offset + (get_sensor_height()/2-
						tmp->img_ver_pos_offset-myacq->cam_param[myacq->cam_idx].hw_roi_param[1]);
				}
				/* binning changes image offset */
				if (myacq->cam_param[myacq->cam_idx].binning > 1) {
					if (myacq->cam_param[myacq->cam_idx].mirror == false) {
						tmp->img_hor_pos_offset = tmp->img_hor_pos_offset + (get_sensor_width()/myacq->cam_param[myacq->cam_idx].binning-
							tmp->img_hor_pos_offset - get_sensor_width()/2/myacq->cam_param[myacq->cam_idx].binning);
					}
					else {
						tmp->img_hor_pos_offset = tmp->img_hor_pos_offset + (get_sensor_width()/myacq->cam_param[myacq->cam_idx].binning-
							tmp->img_hor_pos_offset - get_sensor_width()/2/myacq->cam_param[myacq->cam_idx].binning);
					}
					tmp->img_ver_pos_offset = tmp->img_ver_pos_offset + (get_sensor_height()/myacq->cam_param[myacq->cam_idx].binning-
						tmp->img_ver_pos_offset-get_sensor_height()/2/myacq->cam_param[myacq->cam_idx].binning);
				}

			}

			/* initialize permutation vector */
			if (myacq->cam_param[myacq->cam_idx].mirror)
				myacq->img16[0]->permutation_mirrorh(myacq->cam_param[myacq->cam_idx].img_permutation);
			if (myacq->cam_param[myacq->cam_idx].rotation != 0)
				myacq->img16[0]->permutation_rotation(myacq->cam_param[myacq->cam_idx].rotation, myacq->cam_param[myacq->cam_idx].img_permutation);

			/* support variable for background subtraction */
			myacq->cam_param[myacq->cam_idx].bckimg16 = 
				new ImageProc <unsigned short> (img_height,img_width,(int32_t)myacq->fitNumCores);

			myacq->cam_param[myacq->cam_idx].bckimg16->reset();

			INFO_STREAM << __FUNCTION__ << ", 16-bit image ring buffer initialized" << endl;

		}

		INFO_STREAM << __FUNCTION__ << ", image buffers allocated" << endl;

	}
	catch (...) {
		cout << "acqthread::init_data(): exception" << endl;
	}

	myacq->in_mutex->unlock();

	return TUCSENERR_SUCCESS;

}


void acqthread::delete_data(void)
{
	bool img8_flag = false,
		img16_flag = false;

	myacq->in_mutex->lock();

	try {
		INFO_STREAM << __FUNCTION__ << ", deleting data" << endl;

		if ( !myacq->img8.empty() ) {
			img8_flag = true;
			delete myacq->cam_param[myacq->cam_idx].bckimg8;
		}

		if( !myacq->img16.empty() ) {
			img16_flag = true;
			delete myacq->cam_param[myacq->cam_idx].bckimg16;
		}

		if (myacq->cam_param[myacq->cam_idx].img_permutation) {
			free((void *)myacq->cam_param[myacq->cam_idx].img_permutation);
			myacq->cam_param[myacq->cam_idx].img_permutation = 0;
		}

		/* keep permutation index and background image when disconnected */
		while( !myacq->img8.empty() ) {
			delete myacq->img8.back();
			myacq->img8.pop_back();
		}
		if (img8_flag)
			myacq->img8.clear();

		while( !myacq->img16.empty() ) {
			delete myacq->img16.back();
			myacq->img16.pop_back();
		}
		if (img16_flag)
			myacq->img16.clear();

		INFO_STREAM << __FUNCTION__ << ", data deleted" << endl;

	}
	catch (...) {
		cout << "acqthread::delete_data(): exception" << endl;
	}
	
	myacq->in_mutex->unlock();

}






// manage roi
TUCSENERR acqthread::set_roi(void)
{
	TUCSENERR err = TUCSENERR_SUCCESS;

	INFO_STREAM << __FUNCTION__ << ", initializing ROI" << endl;

	/* check horizontal top left */
	if (myacq->cam_param[myacq->cam_idx].hw_roi_param[0] < 0)
		myacq->cam_param[myacq->cam_idx].hw_roi_param[0] = 0;
	if (myacq->cam_param[myacq->cam_idx].hw_roi_param[0] >= get_sensor_width())
		myacq->cam_param[myacq->cam_idx].hw_roi_param[0] = get_sensor_width()-1;

	/* check vertical top left*/
	if (myacq->cam_param[myacq->cam_idx].hw_roi_param[1] < 0)
		myacq->cam_param[myacq->cam_idx].hw_roi_param[1] = 0;
	if (myacq->cam_param[myacq->cam_idx].hw_roi_param[1] >= get_sensor_width())
		myacq->cam_param[myacq->cam_idx].hw_roi_param[1] = get_sensor_width()-1;

	/* check horizontal bottom right */
	if (myacq->cam_param[myacq->cam_idx].hw_roi_param[2] < myacq->cam_param[myacq->cam_idx].hw_roi_param[0])
		myacq->cam_param[myacq->cam_idx].hw_roi_param[2] = myacq->cam_param[myacq->cam_idx].hw_roi_param[0];
	if (myacq->cam_param[myacq->cam_idx].hw_roi_param[2] >= get_sensor_width())
		myacq->cam_param[myacq->cam_idx].hw_roi_param[2] = get_sensor_width()-1;

	/* check vertical bottom right */
	if (myacq->cam_param[myacq->cam_idx].hw_roi_param[3] < myacq->cam_param[myacq->cam_idx].hw_roi_param[1])
		myacq->cam_param[myacq->cam_idx].hw_roi_param[3] = myacq->cam_param[myacq->cam_idx].hw_roi_param[1];
	if (myacq->cam_param[myacq->cam_idx].hw_roi_param[3] >= get_sensor_height())
		myacq->cam_param[myacq->cam_idx].hw_roi_param[3] = get_sensor_height()-1;

	/* set binning here, at the moment not supported */
	/*if (failed(err = set_binning((int32)myacq->cam_param[myacq->cam_idx].binning))) {
		return err;		
	}*/

	int32_t hw_roi_param[4];
	for (int i = 0; i < 4; i++)
		hw_roi_param[i] = myacq->cam_param[myacq->cam_idx].hw_roi_param[i];

	// effettua la rotazione della ROI
	if (myacq->cam_param[myacq->cam_idx].rotation == 180) {
 		hw_roi_param[0] = get_sensor_width() - myacq->cam_param[myacq->cam_idx].hw_roi_param[2];
 		hw_roi_param[1] = get_sensor_height() - myacq->cam_param[myacq->cam_idx].hw_roi_param[3];
 		hw_roi_param[2] = get_sensor_width() - myacq->cam_param[myacq->cam_idx].hw_roi_param[0];
 		hw_roi_param[3] = get_sensor_height() - myacq->cam_param[myacq->cam_idx].hw_roi_param[1];
		// check the ROI Limits	
		myacq->set_roi_param(hw_roi_param); // these values set the maual_roi_param
		// copy again into temporary variable
		for (int i = 0; i < 4; i++) {
			hw_roi_param[i] = myacq->cam_param[myacq->cam_idx].manual_roi_param[i];
			myacq->cam_param[myacq->cam_idx].hw_roi_param[i] = hw_roi_param[i];
		}

	}

	/* binning won over hw roi */
	if (myacq->cam_param[myacq->cam_idx].binning > 1) {
		myacq->cam_param[myacq->cam_idx].hw_roi = false;
	}

	if (myacq->cam_param[myacq->cam_idx].hw_roi == false) {
		set_offset_hor(0);
		set_offset_ver(0);
		set_width(get_sensor_width() / myacq->cam_param[myacq->cam_idx].binning);
		set_height(get_sensor_height() / myacq->cam_param[myacq->cam_idx].binning);
	}
	else {
		int32_t roi_width = hw_roi_param[2]-hw_roi_param[0]+1;
		int32_t roi_height = hw_roi_param[3]-hw_roi_param[1]+1;

		/* set area dimension */
		myacq->cam_param[myacq->cam_idx].binning = 1;
		set_offset_hor(0);
		set_offset_ver(0);
		set_width(roi_width);
		set_height(roi_height);
		/* set the corner on top/left */
		if (myacq->cam_param[myacq->cam_idx].mirror == false) {
			set_offset_hor(hw_roi_param[0]);
			set_offset_ver(hw_roi_param[1]);
		}
		else {
			int32_t offset_x = get_sensor_width()-(hw_roi_param[0]+roi_width);
				/* avoid overflow */
			if ((roi_width+offset_x) > get_sensor_width()) {
				offset_x = get_sensor_width() - roi_width;
			}
			//offset_x &= ~0x1; // tested for acA1440-73gm
			set_sensor_width(offset_x); /* hw roi mirroring */
			set_sensor_height(hw_roi_param[1]);
		}
	}

	// set HW ROI if binning is disabled
	if (myacq->cam_param[myacq->cam_idx].binning == 1) {
		bool bEnable = true; 
		// set the HW ROI with the current internal roi settings
		if ((err = CameraSetROI(&bEnable, &offset_hor, &offset_ver, &width, &height)) != TUCSENERR_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to set HW ROI" << endl;
			return err;
		}
	}

	INFO_STREAM << __FUNCTION__ << ", ROI initialized" << endl;

	return err;

}


// automatic gain control feedback
bool acqthread::auto_gain_feedback()
{
	int32_t max_value = 0;
	int32_t err = 0;
	bool run_loop = false, deadband_flag = false;

	if ((myacq->cam_param[myacq->cam_idx].gain_auto == true) || 
		   myacq->cam_param[myacq->cam_idx].gain_auto_once)  {
		/* first get the maximum pixel value */
		if (myacq->cam_param[myacq->cam_idx].img_depth == 8)
			max_value = myacq->img8[myacq->img_cur]->max_value;
		else if (myacq->cam_param[myacq->cam_idx].img_depth == 16)
			max_value = myacq->img16[myacq->img_cur]->max_value;
		/* then calculate the error */
		err = myacq->cam_param[myacq->cam_idx].auto_target_value - max_value;
		/* if error over threshold, enable deadband flag */
		if (abs(err) > myacq->cam_param[myacq->cam_idx].auto_feedback_deadband) {
			deadband_flag = true;
		}
		/* if setpoint reached, disable out of deadband flag */
		if (abs(err) <= 1)
			deadband_flag = false;
		/* if out of deadband, run feedback */
		if (deadband_flag)
			run_loop = true;
		else
			run_loop = false;

		/* run once flag gain control (run once for max 30 shots) */
		if (myacq->cam_param[myacq->cam_idx].gain_auto_once) {
			if (myacq->cam_param[myacq->cam_idx].gain_once_cnt++ > 30) {
				myacq->cam_param[myacq->cam_idx].gain_auto_once = false;
				myacq->cam_param[myacq->cam_idx].gain_once_cnt = 0;
			}
			else {
				run_loop = true;
				/* trick to allow run once also when under threshold */
				if (max_value < myacq->cam_param[myacq->cam_idx].auto_feedback_target_thres)
					max_value = myacq->cam_param[myacq->cam_idx].auto_target_value/2+1;
			}
		}
		/* if signal under threshold, pause feedback */
		if (myacq->cam_param[myacq->cam_idx].auto_feedback_target_thres > max_value)
			run_loop = false;
		/* initialize minimum/maximum gain raw */
		if (myacq->cam_param[myacq->cam_idx].gain_auto_cnt == 0) {
			myacq->cam_param[myacq->cam_idx].gain_auto_coeff = 0;
		}
		/* downsampling counter */
		if ((myacq->cam_param[myacq->cam_idx].gain_auto_cnt % (myacq->cam_param[myacq->cam_idx].auto_feedback_dws+1)) != 0) 
			run_loop = false;
		/* run loop */
		if (run_loop) {
			
			int32_t gain_raw, new_gain_raw = 0, 
			gain_raw_d1 = 0, gain_raw_d2 = 0;
			double gain_d1, gain_d2, new_gain;

			/*  XXXYYY
			try {
				gain_raw = myCamera->GainRaw.GetValue();
			}
			catch (GenICam::GenericException &e) {	
				return false;
			}
			catch (...) {
				return false;
			}
			*/			
	
			int32_t delta = 2; /* kick to measure linear coeff gain_raw<->gain_db */
			/* if first loop cycle or error over a fixed threshold */ 
			if ((myacq->cam_param[myacq->cam_idx].gain_auto_coeff == 0) || 
				(abs(myacq->cam_param[myacq->cam_idx].auto_target_value-max_value) > 10)) {
				/* calculate conversion factor between gain_rw and gain_db */
				if ((gain_raw_d1 = (gain_raw+delta)) > myacq->cam_param[myacq->cam_idx].max_raw_gain)
					gain_raw_d1 = myacq->cam_param[myacq->cam_idx].max_raw_gain;				
				gain_d1 = gain_raw_d1;
				if ((gain_raw_d2 = (gain_raw-delta)) < myacq->cam_param[myacq->cam_idx].min_raw_gain)
					gain_raw_d2 = myacq->cam_param[myacq->cam_idx].min_raw_gain;	
				gain_d2 = gain_raw_d2;
				if ((gain_d1-gain_d2) != 0)
					myacq->cam_param[myacq->cam_idx].gain_auto_coeff = 
						(double)(gain_raw_d1-gain_raw_d2)/(gain_d1-gain_d2)*myacq->cam_param[myacq->cam_idx].auto_feedback_gain;
				else
					myacq->cam_param[myacq->cam_idx].gain_auto_coeff = 0;
			}

			/* feedback loop core */
			new_gain_raw = gain_raw;
			if ((max_value > 0) && (myacq->cam_param[myacq->cam_idx].auto_target_value > 0)) {
				if (err > 0) {
					new_gain_raw = gain_raw + (int)(myacq->cam_param[myacq->cam_idx].gain_auto_coeff*
						20*log10((double)myacq->cam_param[myacq->cam_idx].auto_target_value/(double)max_value));
				}
				else if (err < 0) {
					new_gain_raw = gain_raw - (int)(myacq->cam_param[myacq->cam_idx].gain_auto_coeff*
						20*log10((double)max_value/(double)myacq->cam_param[myacq->cam_idx].auto_target_value));
				}
			}

			/* if gain has to be changed */
			if (new_gain_raw != gain_raw) {
				/* set new gain */
				new_gain_raw = new_gain;
				/* check gain feedback limits */
				if (myacq->cam_param[myacq->cam_idx].gain_auto_min < myacq->cam_param[myacq->cam_idx].min_db_gain)
					myacq->cam_param[myacq->cam_idx].gain_auto_min = myacq->cam_param[myacq->cam_idx].min_db_gain;
				if (myacq->cam_param[myacq->cam_idx].gain_auto_max > myacq->cam_param[myacq->cam_idx].max_db_gain)
					myacq->cam_param[myacq->cam_idx].gain_auto_max = myacq->cam_param[myacq->cam_idx].max_db_gain;
			  /* check gain */
				if (new_gain > myacq->cam_param[myacq->cam_idx].gain_auto_max) 
					new_gain = myacq->cam_param[myacq->cam_idx].gain_auto_max;
				else if (new_gain < myacq->cam_param[myacq->cam_idx].gain_auto_min) 
					new_gain = myacq->cam_param[myacq->cam_idx].gain_auto_min;
				if (CameraSetGain(new_gain) == true)
					*myacq->attr_Gain_read = new_gain;
			}
		} /* end if run_loop */
		/* increment loop counter */
		myacq->cam_param[myacq->cam_idx].gain_auto_cnt++;
	}

	return true;

}


// automatic exposure control feedback
bool acqthread::auto_exposure_feedback()
{
	int32_t max_value = 0;
	int32_t err = 0;
	bool run_loop = false, deadband_flag = false;

	if ((myacq->cam_param[myacq->cam_idx].exposure_auto == true) || 
		  myacq->cam_param[myacq->cam_idx].exposure_auto_once) {
		if (myacq->cam_param[myacq->cam_idx].img_depth == 8)
			max_value = myacq->img8[myacq->img_cur]->max_value;
		else if (myacq->cam_param[myacq->cam_idx].img_depth == 16) 
			max_value = myacq->img16[myacq->img_cur]->max_value;
		/* then calculate the error */
		err = myacq->cam_param[myacq->cam_idx].auto_target_value - max_value;
	/* if error over threshold, enable deadband flag */
		if (abs(err) > myacq->cam_param[myacq->cam_idx].auto_feedback_deadband) {
			deadband_flag = true;
		}
		/* if setpoint reached, disable out of deadband flag */
		if (abs(err) <= 1)
			deadband_flag = false;
		/* if out of deadband, run feedback */
		if (deadband_flag)
			run_loop = true;
		else
			run_loop = false;
		/* run once flag gain control (run once for max 30 shots) */
		if (myacq->cam_param[myacq->cam_idx].exposure_auto_once) {
			if (myacq->cam_param[myacq->cam_idx].exposure_once_cnt++ > 30) {
				myacq->cam_param[myacq->cam_idx].exposure_auto_once = false;
				myacq->cam_param[myacq->cam_idx].exposure_once_cnt = 0;
			}
			else {
				run_loop = true;
				/* trick to allow run once also when under threshold */
				if (max_value < myacq->cam_param[myacq->cam_idx].auto_feedback_target_thres)
					max_value = myacq->cam_param[myacq->cam_idx].auto_target_value/2+1;
			}
		}
		/* if signal under threshold, pause feedback */
		if (myacq->cam_param[myacq->cam_idx].auto_feedback_target_thres > max_value)
			run_loop = false;

		/* downsampling counter */
		if ((myacq->cam_param[myacq->cam_idx].exposure_auto_cnt % (myacq->cam_param[myacq->cam_idx].auto_feedback_dws+1)) != 0) 
			run_loop = false;

		/* run loop */
		if (run_loop) {
			int32_t new_exposure = myacq->cam_param[myacq->cam_idx].exposure + 
				(int32_t)((double)(myacq->cam_param[myacq->cam_idx].exposure*err)/(double)max_value*
				myacq->cam_param[myacq->cam_idx].auto_feedback_gain);

			if (new_exposure < myacq->cam_param[myacq->cam_idx].exposure_auto_min) 
				new_exposure = myacq->cam_param[myacq->cam_idx].exposure_auto_min;
			if (new_exposure > myacq->cam_param[myacq->cam_idx].exposure_auto_max) 
				new_exposure = myacq->cam_param[myacq->cam_idx].exposure_auto_max;
			if (CameraSetExposure(new_exposure) == true) {
				*myacq->attr_Exposure_read = new_exposure;
				myacq->attr_Exposure_write = *myacq->attr_Exposure_read;
			}
		} /* end if run_loop */
		/* increment loop counter */
		myacq->cam_param[myacq->cam_idx].exposure_auto_cnt++;
	}

	return true;

}


/*
 * void acqthread::init_timeouts(void)
 * Initialize read param timeouts
 */
void acqthread::init_timeouts(void)
{
	clock_gettime(CLOCK_REALTIME, &roiAttr_time);
	clock_gettime(CLOCK_REALTIME, &tgrAttr_time);
	clock_gettime(CLOCK_REALTIME, &tgrOutAttr_time);
	clock_gettime(CLOCK_REALTIME, &exposure_time);
	clock_gettime(CLOCK_REALTIME, &temperature_time);
	clock_gettime(CLOCK_REALTIME, &gain_time);
	clock_gettime(CLOCK_REALTIME, &data_refresh);

	roiAttr_time.tv_sec -= TUCSENCAM_READ_TOUT;
	tgrAttr_time.tv_sec -= TUCSENCAM_READ_TOUT;
	tgrOutAttr_time.tv_sec -= TUCSENCAM_READ_TOUT;
	exposure_time.tv_sec -= TUCSENCAM_READ_TOUT;
	temperature_time.tv_sec -= TUCSENCAM_READ_TOUT;
	gain_time.tv_sec -= TUCSENCAM_READ_TOUT;
	data_refresh.tv_sec -= TUCSENCAM_READ_TOUT;

}


/*
 * TUCSENERR acqthread::CameraInitAPI()
 * Initialize API, find cameras connection, open connection with the first one
 */
TUCSENERR acqthread::CameraInitAPI(void)
{
	TUCSENERR tucsen_err = TUCSENERR_SUCCESS;
	usleep(10000);

	itApi.pstrConfigPath = NULL;
	itApi.uiCamCount = 0;

	itApi.pstrConfigPath = NULL;
	itApi.uiCamCount = 0;

	INFO_STREAM << __FUNCTION__ << ", initialize API, find cameras " << endl;

	// Initializing SDK API environment
	if ((tucsen_err = TUCAM_Api_Init(&itApi)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", API initialization failure, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl; return tucsen_err;
	}

	if (itApi.uiCamCount == 0) {
		INFO_STREAM << __FUNCTION__ << ", no camera found, initialization failure " << endl;
		return tucsen_err;
	}

	// considering only 1st camera 
	opCam.hIdxTUCam = 0;
	opCam.uiIdxOpen = 0;

	usleep(10000);

	if ((tucsen_err = TUCAM_Dev_Open(&opCam)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to open camera connection, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl; return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", camera found " << opCam.uiIdxOpen << endl;

	return TUCSENERR_SUCCESS;

}


/*
 * TUCSENERR acqthread::CameraInfo()
 * Get camera infos
 */
TUCSENERR acqthread::CameraInfo()
{
	int tucsen_err = TUCSENERR_SUCCESS;
	usleep(10000);

	INFO_STREAM << __FUNCTION__ << ", get camera info" << endl;

	// Get camera info
	TUCAM_VALUE_INFO cameraInfo;

	// usb bus version 
	cameraInfo.nID = TUIDI_BUS;
	if ((tucsen_err = TUCAM_Dev_GetInfo(opCam.hIdxTUCam, &cameraInfo)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to read usb version, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl; return tucsen_err;
	}
	if(cameraInfo.nValue == 0x200 || cameraInfo.nValue == 0x210)
		usb_version = "2.0";
	else
		usb_version = "3.0";
	INFO_STREAM << __FUNCTION__ << ", usb version " << usb_version << endl;

	usleep(10000);

	// camera model 
	cameraInfo.nID = TUIDI_CAMERA_MODEL;
	if ((tucsen_err = TUCAM_Dev_GetInfo(opCam.hIdxTUCam, &cameraInfo)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to read camera model, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl; return tucsen_err;
	}
	camera_model = string(cameraInfo.pText);
	INFO_STREAM << __FUNCTION__ << ", camera model " << camera_model << endl;

	usleep(10000);

	// API version
	cameraInfo.nID = TUIDI_VERSION_API;
	if ((tucsen_err = TUCAM_Dev_GetInfo(opCam.hIdxTUCam, &cameraInfo)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to read API version, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl; return tucsen_err;
	}
	api_version = string(cameraInfo.pText);
	INFO_STREAM << __FUNCTION__ << ", api version " << api_version << endl;

	usleep(10000);

	// Firmware version
	cameraInfo.nID = TUIDI_VERSION_API;
	if ((tucsen_err = TUCAM_Dev_GetInfo(opCam.hIdxTUCam, &cameraInfo)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to read firmware version, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl; return tucsen_err;
	}
	firmware_version = string(cameraInfo.pText);
	INFO_STREAM << __FUNCTION__ << ", firmware version " << firmware_version << endl;

	usleep(10000);

	// Serial number
	char snVal[64] = {0};
	TUCAM_REG_RW regRW;
	regRW.nBufSize = 64;
	regRW.pBuf = snVal;
	regRW.nRegType = TUREG_SN;
	if ((tucsen_err =  TUCAM_Reg_Read(opCam.hIdxTUCam, regRW)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to read serial number, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl; return tucsen_err;
	}
	serial_number = string(snVal);
	INFO_STREAM << __FUNCTION__ << ", serial number " << serial_number << endl;

	usleep(10000);

	// get min/max exposure 
	expProp.idProp = TUIDP_EXPOSURETM;
	if ((tucsen_err = TUCAM_Prop_GetAttr(opCam.hIdxTUCam, &expProp)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get min/max exposure, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		//return tucsen_err;
	}
	min_exposure = expProp.dbValMin; max_exposure = expProp.dbValMax;
	INFO_STREAM << __FUNCTION__ << ", min/max exposure " << min_exposure << "/" << max_exposure << endl;

	usleep(10000);

	// get min/max temperature (Celsius)
	expProp.idProp = TUIDP_TEMPERATURE;
	if ((tucsen_err = TUCAM_Prop_GetAttr(opCam.hIdxTUCam, &tempProp)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get min/max temperature, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		//return tucsen_err;
	}
	min_temperature = tempProp.dbValMin - tempOffset; max_temperature = tempProp.dbValMax - tempOffset;
	INFO_STREAM << __FUNCTION__ << ", min/max temperature " << min_temperature << "/" << max_temperature << endl;

	usleep(10000);

	// get min/max gain 
	gainProp.idProp = TUIDP_GLOBALGAIN;
	if ((tucsen_err = TUCAM_Prop_GetAttr(opCam.hIdxTUCam, &gainProp)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get min/max gain, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		//return tucsen_err;
	}
	min_gain = gainProp.dbValMin; max_gain = gainProp.dbValMax;
	INFO_STREAM << __FUNCTION__ << ", min/max gain " << min_gain << "/" << max_gain << endl;

	usleep(10000);

	tucsen_err = TUCSENERR_SUCCESS;

	return tucsen_err;
}


/*
 * TUCSENERR acqthread::CameraSoftTrigger()
 * Send a software trigger to the camera
 */
TUCSENERR acqthread::CameraSoftTrigger()
{
	int tucsen_err = TUCSENERR_SUCCESS;

	INFO_STREAM << __FUNCTION__ << ", send soft trigger" << endl;

	if ((tucsen_err = TUCAM_Cap_DoSoftwareTrigger(opCam.hIdxTUCam)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to send soft trigger, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraEnableHwTrigger()
 * Send a software trigger to the camera
 */
TUCSENERR acqthread::CameraEnableHwTrigger(bool hw_trigger)
{
	int tucsen_err = TUCSENERR_SUCCESS;

	usleep(10000);

	int rd_nTgrMode, wr_nTgrMode;

	INFO_STREAM << __FUNCTION__ << ", set hw trigger mode " <<  hw_trigger << endl;

	// read actual trigger mode setting
	if ((tucsen_err = CameraGetTriggerMode(true, &rd_nTgrMode, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get trigger mode, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	// if requesting free running trigger but HW configured
	if ((hw_trigger == false) && (rd_nTgrMode != TUCCM_SEQUENCE)) {
		wr_nTgrMode = TUCCM_SEQUENCE;
		if ((tucsen_err = CameraSetTriggerMode(&wr_nTgrMode, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to set sw trigger mode, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		}
	}
	// if requesting HW trigger but free running configured
	else if ((hw_trigger == true) && (rd_nTgrMode == TUCCM_SEQUENCE)) {	
		// set standard sequence trigger by default
		wr_nTgrMode = TUCCM_TRIGGER_STANDARD;
		if ((tucsen_err = CameraSetTriggerMode(&wr_nTgrMode, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to set hw trigger mode, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		}
	}

	return tucsen_err;	

}


/*
 * TUCSENERR acqthread::CameraGetTriggerMode()
 * Get trigger mode attributes
 *
 * INT32   nTgrMode;  [in/out] The mode of trigger 
 *  		TUCCM_SEQUENCE              = 0x00,   // capture start sequence mode (free running)
 *  		TUCCM_TRIGGER_STANDARD      = 0x01,   // capture start trigger standard mode
 *  		TUCCM_TRIGGER_SYNCHRONOUS   = 0x02,   // capture start trigger synchronous mode
 *  		TUCCM_TRIGGER_GLOBAL        = 0x03,   // capture start trigger global
 *  		TUCCM_TRIGGER_SOFTWARE      = 0x04,   // capture start trigger software
 * INT32   nExpMode;  [in/out] The mode of exposure [0, 1] 0:Exposure time   1:Width level 
 * INT32   nEdgeMode; [in/out] The mode of edge     [0, 1] 0:Falling edge    1:Rising edge
 * INT32   nDelayTm;  [in/out] The time delay
 * INT32   nFrames;   [in/out] How many frames per trigger
*/
TUCSENERR acqthread::CameraGetTriggerMode(bool force_read, int *rd_nTgrMode, int *rd_nExpMode, 
	int *rd_nEdgeMode, int *rd_nDelayTm, int *rd_nFrames)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	struct timespec now;

	usleep(10000);
	
	clock_gettime(CLOCK_REALTIME, &now);

	// return cached values
	if ((myacq->difftimems(tgrAttr_time,now) < TUCSENCAM_READ_TOUT) && !force_read) {
		// update not-NULL fields 
		if (rd_nTgrMode != NULL)
			*rd_nTgrMode = tgrAttr.nTgrMode;
		if (rd_nExpMode != NULL)
			*rd_nExpMode = tgrAttr.nExpMode;
		if (rd_nEdgeMode != NULL)
			*rd_nEdgeMode = tgrAttr.nEdgeMode;	
		if (rd_nDelayTm != NULL)
			*rd_nDelayTm = tgrAttr.nDelayTm;
		if (rd_nFrames != NULL)
			*rd_nFrames = tgrAttr.nFrames;	

		return tucsen_err;
	}


	INFO_STREAM << __FUNCTION__ << " get trigger mode attributes" << endl;

	if ((tucsen_err = TUCAM_Cap_GetTrigger(opCam.hIdxTUCam, &tgrAttr)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get trigger mode attributes, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	

	INFO_STREAM << __FUNCTION__ << ", trigger mode " << tgrAttr.nTgrMode << ", " << 
		GetTUCSENString(TUCSEN_TRIG_MODE_map, tgrAttr.nTgrMode) << endl;
 
	INFO_STREAM << __FUNCTION__ << ", trigger exposure mode " << tgrAttr.nExpMode << ", " << 
		GetTUCSENString(TUCSEN_TRIGGER_EXP_map, tgrAttr.nExpMode) << endl; 

	INFO_STREAM << __FUNCTION__ << ", trigger edge mode " << tgrAttr.nEdgeMode << ", " << 
		GetTUCSENString(TUCSEN_TRIGGER_EDGE_map, tgrAttr.nEdgeMode) << endl; 

	INFO_STREAM << __FUNCTION__ << ", trigger time delay " << tgrAttr.nDelayTm << " us" << endl; 
	INFO_STREAM << __FUNCTION__ << ", frames per trigger " << tgrAttr.nFrames << endl; 

	clock_gettime(CLOCK_REALTIME, &tgrAttr_time);

	// update not-NULL fields 
	if (rd_nTgrMode != NULL)
		*rd_nTgrMode = tgrAttr.nTgrMode;
	if (rd_nExpMode != NULL)
		*rd_nExpMode = tgrAttr.nExpMode;
	if (rd_nEdgeMode != NULL)
		*rd_nEdgeMode = tgrAttr.nEdgeMode;	
	if (rd_nDelayTm != NULL)
		*rd_nDelayTm = tgrAttr.nDelayTm;
	if (rd_nFrames != NULL)
		*rd_nFrames = tgrAttr.nFrames;

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraSetTriggerMode()
 * Set trigger mode attributes
 *
 * INT32   nTgrMode;  [in/out] The mode of trigger 
 *  		TUCCM_SEQUENCE              = 0x00,   // capture start sequence mode
 *  		TUCCM_TRIGGER_STANDARD      = 0x01,   // capture start trigger standard mode
 *  		TUCCM_TRIGGER_SYNCHRONOUS   = 0x02,   // capture start trigger synchronous mode
 *  		TUCCM_TRIGGER_GLOBAL        = 0x03,   // capture start trigger global
 *  		TUCCM_TRIGGER_SOFTWARE      = 0x04,   // capture start trigger software
 * INT32   nExpMode;  [in/out] The mode of exposure [0, 1] 0:Exposure time   1:Width level 
 * INT32   nEdgeMode; [in/out] The mode of edge     [0, 1] 0:Falling edge    1:Rising edge
 * INT32   nDelayTm;  [in/out] The time delay
 * INT32   nFrames;   [in/out] How many frames per trigger
*/
TUCSENERR acqthread::CameraSetTriggerMode(int *nTgrMode, int *nExpMode, int *nEdgeMode, int *nDelayTm, int *nFrames)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	TUCAM_TRIGGER_ATTR wr_tgrAttr;

	usleep(10000);

	struct timespec now;
	clock_gettime(CLOCK_REALTIME, &now);

	
	// if data is not expired use the internal buffered values
	if (myacq->difftimems(tgrAttr_time,now) > TUCSENCAM_READ_TOUT) {
		if ((tucsen_err = CameraGetTriggerMode(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to get trigger mode attributes" << endl;	
			return tucsen_err;	
		}	
	}

	// copy data into wr structure
	memcpy(&wr_tgrAttr, &tgrAttr, sizeof(TUCAM_TRIGGER_ATTR));
	
	// update not-NULL fields 
	if (nTgrMode != NULL)
		wr_tgrAttr.nTgrMode = *nTgrMode;
	if (nExpMode != NULL)
		wr_tgrAttr.nExpMode = *nExpMode;
	if (nEdgeMode != NULL)
		wr_tgrAttr.nEdgeMode = *nEdgeMode;	
	if (nDelayTm != NULL)
		wr_tgrAttr.nDelayTm = *nDelayTm;
	if (nFrames != NULL)
		wr_tgrAttr.nFrames = *nFrames;	

	INFO_STREAM << __FUNCTION__ << " set trigger mode attributes, " << " mode " << wr_tgrAttr.nTgrMode << endl;

	if ((tucsen_err = TUCAM_Cap_SetTrigger(opCam.hIdxTUCam, wr_tgrAttr)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to set trigger mode attributes, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", trigger mode " << wr_tgrAttr.nTgrMode << ", " << 
		GetTUCSENString(TUCSEN_TRIG_MODE_map, wr_tgrAttr.nTgrMode) << endl;
 
	INFO_STREAM << __FUNCTION__ << ", trigger exposure mode " << wr_tgrAttr.nExpMode << ", " << 
		GetTUCSENString(TUCSEN_TRIGGER_EXP_map, wr_tgrAttr.nExpMode) << endl; 

	INFO_STREAM << __FUNCTION__ << ", trigger edge mode " << wr_tgrAttr.nEdgeMode << ", " << 
		GetTUCSENString(TUCSEN_TRIGGER_EDGE_map, wr_tgrAttr.nEdgeMode) << endl; 

	INFO_STREAM << __FUNCTION__ << ", trigger time delay " << wr_tgrAttr.nDelayTm << " ns" << endl; 
	INFO_STREAM << __FUNCTION__ << ", frames per trigger " << wr_tgrAttr.nFrames << endl; 

	tgrAttr_time.tv_sec -= TUCSENCAM_READ_TOUT*2; // force read refresh after the setting

	// if data is not expired use the internal buffered values
	if (myacq->difftimems(tgrAttr_time,now) > TUCSENCAM_READ_TOUT) {
		if ((tucsen_err = CameraGetTriggerMode(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to get trigger mode attributes" << endl;	
			return tucsen_err;	
		}	
	}

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraGetTriggerOut()
 * Get trigger out attributes
 *
 * INT32   nTgrOutPort;   [in/out] The port of trigger out (1,2,3)
 * INT32   nTgrOutMode;   [in/out] The mode of trigger out
 *	TUOPT_GND        = 0x00,      // use low 
 *	TUOPT_VCC        = 0x01,      // use high
 *	TUOPT_IN         = 0x02,      // use trigger input
 *	TUOPT_EXPSTART   = 0x03,      // use exposure start
 *	TUOPT_EXPGLOBAL  = 0x04,      // use global exposure 
 *	TUOPT_READEND    = 0x05,      // use read end
 * INT32   nEdgeMode;     [in/out] The mode of edge   [0, 1] 1:Falling edge  0:Rising edge
 * INT32   nDelayTm;      [in/out] The time delay
 * INT32   nWidth;        [in/out] The width of pulse
 */
TUCSENERR acqthread::CameraGetTriggerOut(bool force_read, int *rd_nTgrOutPort, int *rd_nTgrOutMode, 
	int *rd_nEdgeMode, int *rd_nDelayTm, int *rd_nWidth)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	struct timespec now;

	usleep(10000);
	
	clock_gettime(CLOCK_REALTIME, &now);

	// return cached values
	if ((myacq->difftimems(tgrOutAttr_time,now) < TUCSENCAM_READ_TOUT) && !force_read) {
		if (rd_nTgrOutPort != NULL)
			*rd_nTgrOutPort = tgrOutAttr.nTgrOutPort;
		if (rd_nTgrOutMode != NULL)
			*rd_nTgrOutMode = tgrOutAttr.nTgrOutMode;
		if (rd_nEdgeMode != NULL)
			*rd_nEdgeMode = tgrOutAttr.nEdgeMode;
		if (rd_nDelayTm != NULL)
			*rd_nDelayTm = tgrOutAttr.nDelayTm;	
		if (rd_nWidth != NULL)
			*rd_nWidth = tgrOutAttr.nWidth;			
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << " get trigger out attributes" << endl;

	if ((tucsen_err = TUCAM_Cap_GetTriggerOut(opCam.hIdxTUCam, &tgrOutAttr)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get trigger out attributes, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", trigger out port " << tgrOutAttr.nTgrOutPort << ", " << 
		GetTUCSENString(TUCSEN_OUTPUTTRG_PORT_map, tgrOutAttr.nTgrOutPort) << endl;

	INFO_STREAM << __FUNCTION__ << ", trigger out mode " << tgrOutAttr.nTgrOutMode << ", " << 
		GetTUCSENString(TUCSEN_OUTPUTTRG_KIND_map, tgrOutAttr.nTgrOutMode) << endl; 

	INFO_STREAM << __FUNCTION__ << ", trigger out edge " << tgrOutAttr.nEdgeMode << ", " << 
		GetTUCSENString(TUCSEN_OUTPUTTRG_EDGE_map, tgrOutAttr.nEdgeMode) << endl; 

	INFO_STREAM << __FUNCTION__ << ", trigger out time delay " << tgrOutAttr.nDelayTm << " ns" << endl;

	INFO_STREAM << __FUNCTION__ << ", trigger out width " << tgrOutAttr.nWidth << " ns" << endl;

	clock_gettime(CLOCK_REALTIME, &tgrOutAttr_time);

	if (rd_nTgrOutPort != NULL)
		*rd_nTgrOutPort = tgrOutAttr.nTgrOutPort;
	if (rd_nTgrOutMode != NULL)
		*rd_nTgrOutMode = tgrOutAttr.nTgrOutMode;
	if (rd_nEdgeMode != NULL)
		*rd_nEdgeMode = tgrOutAttr.nEdgeMode;
	if (rd_nDelayTm != NULL)
		*rd_nDelayTm = tgrOutAttr.nDelayTm;	
	if (rd_nWidth != NULL)
		*rd_nWidth = tgrOutAttr.nWidth;	

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraSetTriggeOut()
 * Set trigger mode attributes
 *
 * INT32   nTgrOutPort;   [in/out] The port of trigger out (1,2,3)
 * INT32   nTgrOutMode;   [in/out] The mode of trigger out
 * INT32   nEdgeMode;     [in/out] The mode of edge   [0, 1] 1:Falling edge  0:Rising edge
 * INT32   nDelayTm;      [in/out] The time delay
 * INT32   nWidth;        [in/out] The width of pulse
 */
TUCSENERR acqthread::CameraSetTriggerOut(int *nTgrOutPort, int *nTgrOutMode, int *nEdgeMode, int *nDelayTm, int *nWidth)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	TUCAM_TRGOUT_ATTR wr_tgrOutAttr;

	usleep(10000);

	struct timespec now;
	clock_gettime(CLOCK_REALTIME, &now);

	// if data is not expired use the internal buffered values
	if (myacq->difftimems(tgrOutAttr_time,now) > TUCSENCAM_READ_TOUT) {
		if ((tucsen_err = CameraGetTriggerOut(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to get trigger out attributes" << endl;	
			return tucsen_err;	
		}	
	}

	// copy data into wr structure
	memcpy(&wr_tgrOutAttr, &tgrOutAttr, sizeof(TUCAM_TRGOUT_ATTR));
	
	// update not-NULL fields 
	if (nTgrOutPort != NULL)
		wr_tgrOutAttr.nTgrOutPort = *nTgrOutPort;
	if (nTgrOutMode != NULL)
		wr_tgrOutAttr.nTgrOutMode = *nTgrOutMode;
	if (nEdgeMode != NULL)
		wr_tgrOutAttr.nEdgeMode = *nEdgeMode;	
	if (nDelayTm != NULL)
		wr_tgrOutAttr.nDelayTm = *nDelayTm;
	if (nWidth != NULL)
		wr_tgrOutAttr.nWidth = *nWidth;	

	INFO_STREAM << __FUNCTION__ << " set trigger out attributes" << endl;

	if ((tucsen_err = TUCAM_Cap_SetTriggerOut(opCam.hIdxTUCam, wr_tgrOutAttr)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to set trigger out attributes, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", trigger out port " << wr_tgrOutAttr.nTgrOutPort << ", " << 
		GetTUCSENString(TUCSEN_OUTPUTTRG_PORT_map, wr_tgrOutAttr.nTgrOutPort) << endl;

	INFO_STREAM << __FUNCTION__ << ", trigger out mode " << wr_tgrOutAttr.nTgrOutMode << ", " << 
		GetTUCSENString(TUCSEN_OUTPUTTRG_KIND_map, wr_tgrOutAttr.nTgrOutMode) << endl; 

	INFO_STREAM << __FUNCTION__ << ", trigger out edge " << wr_tgrOutAttr.nEdgeMode << ", " << 
		GetTUCSENString(TUCSEN_TRIGGER_EDGE_map, wr_tgrOutAttr.nEdgeMode) << endl; 

	INFO_STREAM << __FUNCTION__ << ", trigger out time delay " << wr_tgrOutAttr.nDelayTm << " ns" << endl;

	INFO_STREAM << __FUNCTION__ << ", trigger out width " << wr_tgrOutAttr.nWidth << " ns" << endl;

	tgrOutAttr_time.tv_sec -= TUCSENCAM_READ_TOUT*2; // force read refresh after the setting

	// if data is not expired use the internal buffered values
	if (myacq->difftimems(tgrOutAttr_time,now) > TUCSENCAM_READ_TOUT) {
		if ((tucsen_err = CameraGetTriggerOut(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to get trigger out attributes" << endl;	
			return tucsen_err;	
		}	
	}

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraGetROI()
 * Get camera ROI attributes
 *
 * BOOL    bEnable;   [in/out] The ROI enable
 * INT32   nHOffset;  [in/out] The horizontal offset
 * INT32   nVOffset;  [in/out] The vertical offset
 * INT32   nWidth;    [in/out] The ROI width
 * INT32   nHeight;   [in/out] The ROI height
 */
TUCSENERR acqthread::CameraGetROI(bool force_read, bool *bEnable, int *nHOffset, int *nVOffset, int *nWidth, int *nHeight)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	struct timespec now;

	usleep(10000);
	
	clock_gettime(CLOCK_REALTIME, &now);

	if ((myacq->difftimems(roiAttr_time,now) < TUCSENCAM_READ_TOUT) && !force_read) {
		if (bEnable != NULL)
			*bEnable = roiAttr.bEnable;
		if (nHOffset != NULL)
			*nHOffset = roiAttr.nHOffset;		
		if (nVOffset != NULL)
			*nVOffset = roiAttr.nVOffset;
		if (nWidth != NULL)
			*nWidth = roiAttr.nWidth;
		if (nHeight != NULL)
			*nHeight = roiAttr.nHeight;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << " get ROI attributes" << endl;

	if ((tucsen_err = TUCAM_Cap_GetROI(opCam.hIdxTUCam, &roiAttr)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get ROI attributes, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", ROI enabled " << roiAttr.bEnable << endl;
	INFO_STREAM << __FUNCTION__ << ", ROI horizontal offset " << roiAttr.nHOffset << endl; 
	INFO_STREAM << __FUNCTION__ << ", ROI vertical offset " << roiAttr.nVOffset << endl; 
	INFO_STREAM << __FUNCTION__ << ", ROI width " << roiAttr.nWidth << endl; 
	INFO_STREAM << __FUNCTION__ << ", ROI height " << roiAttr.nHeight << endl; 

	clock_gettime(CLOCK_REALTIME, &roiAttr_time);

	if (bEnable != NULL)
		*bEnable = roiAttr.bEnable;
	if (nHOffset != NULL)
		*nHOffset = roiAttr.nHOffset;		
	if (nVOffset != NULL)
		*nVOffset = roiAttr.nVOffset;
	if (nWidth != NULL)
		*nWidth = roiAttr.nWidth;
	if (nHeight != NULL)
		*nHeight = roiAttr.nHeight;

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraSetROI()
 * Set camera ROI attributes
 *
 * BOOL    bEnable;   [in/out] The ROI enable
 * INT32   nHOffset;  [in/out] The horizontal offset
 * INT32   nVOffset;  [in/out] The vertical offset
 * INT32   nWidth;    [in/out] The ROI width
 * INT32   nHeight;   [in/out] The ROI height
 */
TUCSENERR acqthread::CameraSetROI(bool *bEnable, int *nHOffset, int *nVOffset, int *nWidth, int *nHeight)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	TUCAM_ROI_ATTR wr_roiAttr;

	usleep(10000);

	struct timespec now;
	clock_gettime(CLOCK_REALTIME, &now);

	// if data is not expired use the internal buffered values
	if (myacq->difftimems(roiAttr_time,now) > TUCSENCAM_READ_TOUT) {
		if ((tucsen_err = CameraGetROI(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to get ROI attributes" << endl;	
			return tucsen_err;	
		}	
	}

	// copy data into wr structure
	memcpy(&wr_roiAttr, &roiAttr, sizeof(TUCAM_ROI_ATTR));
	
	// update not-NULL fields 
	if (bEnable != NULL)
		wr_roiAttr.bEnable = *bEnable;
	if (nHOffset != NULL)
		wr_roiAttr.nHOffset = *nHOffset;
	if (nVOffset != NULL)
		wr_roiAttr.nVOffset = *nVOffset;	
	if (nWidth != NULL)
		wr_roiAttr.nWidth = *nWidth;
	if (nHeight != NULL)
		wr_roiAttr.nHeight = *nHeight;	

	INFO_STREAM << __FUNCTION__ << " set ROI attributes" << endl;

	if ((tucsen_err = TUCAM_Cap_SetROI(opCam.hIdxTUCam, wr_roiAttr)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to set ROI attributes, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", ROI enabled " << wr_roiAttr.bEnable << endl;
	INFO_STREAM << __FUNCTION__ << ", ROI horizontal offset " << wr_roiAttr.nHOffset << endl; 
	INFO_STREAM << __FUNCTION__ << ", ROI vertical offset " << wr_roiAttr.nVOffset << endl; 
	INFO_STREAM << __FUNCTION__ << ", ROI width " << wr_roiAttr.nWidth << endl; 
	INFO_STREAM << __FUNCTION__ << ", ROI height " << wr_roiAttr.nHeight << endl; 

	roiAttr_time.tv_sec -= TUCSENCAM_READ_TOUT*2; // force read refresh after the setting

	// if data is not expired use the internal buffered values
	if (myacq->difftimems(roiAttr_time,now) > TUCSENCAM_READ_TOUT) {
		if ((tucsen_err = CameraGetROI(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to get ROI attributes" << endl;	
			return tucsen_err;	
		}	
	}


	return tucsen_err;
}


/*
 * TUCSENERR acqthread::CameraResetROI()
 * Disable camera hardware ROI
 *
 */
TUCSENERR acqthread::CameraResetROI(void)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	struct timespec now;

	usleep(10000);
	
	clock_gettime(CLOCK_REALTIME, &now);

	INFO_STREAM << __FUNCTION__ << ", reset ROI" << endl; 

	bool bEnable = false;
	if ((tucsen_err = CameraSetROI(&bEnable, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to reset ROI" << endl;
		return tucsen_err;
	}

	roiAttr_time.tv_sec -= TUCSENCAM_READ_TOUT*2; // force read refresh after the setting

	// if data is not expired use the internal buffered values
	if (myacq->difftimems(roiAttr_time,now) > TUCSENCAM_READ_TOUT) {
		if ((tucsen_err = CameraGetROI(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
			INFO_STREAM << __FUNCTION__ << ", failed to get ROI attributes" << endl;	
			return tucsen_err;	
		}	
	}

	return tucsen_err;

}

/*
 * TUCSENERR acqthread::CameraGetExposure()
 * Get camera exposure (ms)
 */
TUCSENERR acqthread::CameraGetExposure(bool force_read, double *rd_exposure)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	struct timespec now;

	usleep(10000);
	
	clock_gettime(CLOCK_REALTIME, &now);

	if ((myacq->difftimems(exposure_time,now) < TUCSENCAM_READ_TOUT) && !force_read) {
		if (rd_exposure != NULL)
			*rd_exposure = exposure; // returns cached value
		return tucsen_err;
	}

	double tmp;

	if ((tucsen_err = TUCAM_Prop_GetValue(opCam.hIdxTUCam, TUIDP_EXPOSURETM, &tmp)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get exposure, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	exposure = tmp * 1000;

	clock_gettime(CLOCK_REALTIME, &exposure_time);

	if (rd_exposure != NULL)
		*rd_exposure = exposure;

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraSetExposure()
 * Set camera exposure (ms)
 */
TUCSENERR acqthread::CameraSetExposure(double wr_exposure)
{
	int tucsen_err = TUCSENERR_SUCCESS;

	usleep(10000);

	INFO_STREAM << __FUNCTION__ << " set camera exposure" << endl;

	

	if ((tucsen_err = TUCAM_Prop_SetValue(opCam.hIdxTUCam, TUIDP_EXPOSURETM, wr_exposure/1000)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to set exposure, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", set camera exposure " << wr_exposure << endl;
	exposure = wr_exposure;

	exposure_time.tv_sec -= TUCSENCAM_READ_TOUT*2; // force read refresh after the setting

	

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraGetTemperature()
 * Get camera temperature (C)
 */
TUCSENERR acqthread::CameraGetTemperature(bool force_read, double *rd_temperature)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	struct timespec now;

	usleep(10000);

	clock_gettime(CLOCK_REALTIME, &now);

	if ((myacq->difftimems(temperature_time,now) < TUCSENCAM_TEMPERATURE_TIMEOUT) && !force_read) {
		if (rd_temperature != NULL)
			*rd_temperature = temperature; // returns cached value
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << " get camera temperature" << endl;

	if ((tucsen_err = TUCAM_Prop_GetValue(opCam.hIdxTUCam, TUIDP_TEMPERATURE, &temperature)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get camera temperature, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", camera temperature " << temperature << endl;

	clock_gettime(CLOCK_REALTIME, &temperature_time);

	if (rd_temperature != NULL)
		*rd_temperature = temperature;

	return tucsen_err;

}

/*
 * TUCSENERR acqthread::CameraSetTemperature()
 * Set camera temperature (C)
 */
TUCSENERR acqthread::CameraSetTemperature(double wr_temperature)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	INFO_STREAM << __FUNCTION__ << " set camera temperature" << endl;

	usleep(10000);

	// temperature value is an index here
	// library maps -50:50C to 0:100...tempOffset = 50
	if ((tucsen_err = TUCAM_Prop_SetValue(opCam.hIdxTUCam, TUIDP_TEMPERATURE, (int)wr_temperature + tempOffset)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to set exposure, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	// store the temperature setpoint
	temperature_setpoint = (int)wr_temperature;

	*myacq->attr_TemperatureSetpoint_read = temperature_setpoint;

	INFO_STREAM << __FUNCTION__ << ", set camera temperature " << wr_temperature << endl;

	temperature_time.tv_sec -= TUCSENCAM_READ_TOUT; // force read refresh after the setting

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraGetTemperatureSetpoint()
 * Get temperature setpoint
 */
TUCSENERR acqthread::CameraGetTemperatureSetpoint(bool force_read, double *rd_temperature)
{
	int tucsen_err = TUCSENERR_SUCCESS;

	if (rd_temperature != NULL)
		*rd_temperature = temperature_setpoint;

	INFO_STREAM << __FUNCTION__ << ", camera temperature setpoint " << temperature_setpoint << endl;

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraGetGain()
 * Get camera gain
 * (usually 0 - 1x, 1 - 2x, 2 - 4x)
 */
TUCSENERR acqthread::CameraGetGain(bool force_read, double *rd_gain)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	struct timespec now;

	usleep(10000);
	
	clock_gettime(CLOCK_REALTIME, &now);

	if ((myacq->difftimems(gain_time,now) < TUCSENCAM_READ_TOUT) && !force_read) {
		if (rd_gain != NULL)
			*rd_gain = gain; // returns cached value
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << " get camera gain" << endl;

	if ((tucsen_err = TUCAM_Prop_GetValue(opCam.hIdxTUCam, TUIDP_GLOBALGAIN, &gain)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to get gain, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", camera gain " << gain << endl;

	clock_gettime(CLOCK_REALTIME, &gain_time);

	if (rd_gain != NULL)
		*rd_gain = gain; // returns cached value

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraSetGain()
 * Set camera gain
 * (usually 0 - 1x, 1 - 2x, 2 - 4x)
 */
TUCSENERR acqthread::CameraSetGain(double wr_gain)
{
	int tucsen_err = TUCSENERR_SUCCESS;

	usleep(10000);

	INFO_STREAM << __FUNCTION__ << " set camera gain" << endl;

	if ((tucsen_err = TUCAM_Prop_SetValue(opCam.hIdxTUCam, TUIDP_GLOBALGAIN, wr_gain)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to set gain, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	gain = wr_gain;
	INFO_STREAM << __FUNCTION__ << ", set camera gain " << wr_gain << endl;

	gain_time.tv_sec -= TUCSENCAM_READ_TOUT; // force read refresh after the setting

	return tucsen_err;

}

/*
 * string acqthread::CameraGetInfo()
 * It returns a string containing: usb_version, api_version, 
 *		                   firmware_version, camera_model, 
 *                                 camera_name, serial_number;
 */
string acqthread::CameraGetInfo(void)
{
	string info;

	info += "usb version:         " + usb_version + '\n';
	info += "API version:         " + api_version + '\n';
	info += "fw version:          " + firmware_version + '\n';
	info += "camera model:        " + camera_model + '\n';
	info += "camera name:         " + camera_name + '\n';
	info += "s/n:                 " + serial_number + '\n';
	info += "min/max exposure:    " + to_string(min_exposure*1000) + " / " + to_string(max_exposure*1000) + " us" + '\n';
	info += "min/max gain:        " + to_string(min_gain) + " / " + to_string(max_gain) + '\n';
	info += "min/max temperature: " + to_string(min_temperature) + " / " + to_string(max_temperature) + " C" + '\n';

	return info;

}


/*
 * string acqthread::CameraGetTriggerInfo()
 * It returns a string containing:  nTgrMode, nExpMode, nEdgeMode,
 *                                  nDelayTm, nFrames 
 */
string acqthread::CameraGetTriggerInfo(void)
{
	string info;

	info += "trigger mode:       " + GetTUCSENString(TUCSEN_TRIG_MODE_map, tgrAttr.nTgrMode) + '\n';
	info += "exposure mode:      " + GetTUCSENString(TUCSEN_TRIGGER_EXP_map, tgrAttr.nExpMode) + '\n';
	info += "edge mode:          " + GetTUCSENString(TUCSEN_TRIGGER_EDGE_map, tgrAttr.nEdgeMode) + '\n';
	info += "trigger delay:      " + to_string(tgrAttr.nDelayTm) + " us" + '\n';
	info += "frames per trigger: " + to_string(tgrAttr.nFrames) + '\n';

	return info;

}


/*
 * string acqthread::CameraGetTriggerInfo()
 * It returns a string containing:  nTgrMode, nExpMode, nEdgeMode,
 *                                  nDelayTm, nFrames 
 */
string acqthread::CameraGetTriggerOutInfo(void)
{
	string info;

	info += "trigger port:  " + GetTUCSENString(TUCSEN_OUTPUTTRG_PORT_map, tgrOutAttr.nTgrOutPort) + '\n';
	info += "trigger mode:  " + GetTUCSENString(TUCSEN_OUTPUTTRG_KIND_map, tgrOutAttr.nTgrOutMode) + '\n';
	info += "edge mode:     " + GetTUCSENString(TUCSEN_TRIGGER_EDGE_map, tgrOutAttr.nEdgeMode) + '\n';
	info += "trigger delay: " + to_string(tgrOutAttr.nDelayTm) + " us" + '\n';
	info += "trigger width: " + to_string(tgrOutAttr.nWidth) + " us" + '\n';

	return info;

}


/*
 * TUCSENERR acqthread::CameraInit(void)
 * Initialize camera parameters 
 */
TUCSENERR acqthread::CameraInit(void)
{
	int tucsen_err = TUCSENERR_SUCCESS;
	bool bEnable;

	usleep(10000);

	INFO_STREAM << __FUNCTION__ << " initialize camera parameters" << endl;	

	myacq->cam_param[myacq->cam_idx].configured = false;

	// Reset ROI to get the full dimension of the CMOS sensor
	if ((tucsen_err = CameraResetROI()) != TUCAMRET_SUCCESS) {	
		return tucsen_err;
	}

	// Initialize sensor dimensions 
	if ((tucsen_err = CameraGetROI(true, &bEnable, &offset_hor, &offset_ver, &sensor_width, &sensor_height)) != TUCAMRET_SUCCESS) {
		return tucsen_err;
	}
	width = sensor_width;
	height = sensor_height;

	// Initialize trigger mode internal variables
	if ((tucsen_err =  CameraGetTriggerMode(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
		return tucsen_err;
	}

	// Initialize trigger out internal variables
	if ((tucsen_err =  CameraGetTriggerOut(true, NULL, NULL, NULL, NULL, NULL)) != TUCAMRET_SUCCESS) {
		return tucsen_err;
	}	

	temperature_setpoint = 0;

	/**** HERE INITIALIZE CAMERA PARAMETERS (binning,roi,imagedepth excluded) ****/

	// AT THIS STAGE IGNORE ERRORS DURING INITIALIZATION

	
	/* trigger parameters */
	int nTgrMode = myacq->cam_param[myacq->cam_idx].trigger_mode,
	    nExpMode = myacq->cam_param[myacq->cam_idx].trigger_expmode, 
	    nEdgeMode = myacq->cam_param[myacq->cam_idx].trigger_edgemode,
	    nDelayTm = myacq->cam_param[myacq->cam_idx].trigger_delaytime,
	    nFrames = 1;

	CameraSetTriggerMode(&nTgrMode, &nExpMode, &nEdgeMode, &nDelayTm, &nFrames);
	
	/* trigger out parameters */
	int nTgrOutPort = myacq->cam_param[myacq->cam_idx].triggerout_port,
	    nTgrOutMode = myacq->cam_param[myacq->cam_idx].triggerout_mode,
	    nEdgeOutMode = myacq->cam_param[myacq->cam_idx].triggerout_edgemode,
	    nDelayOutTm = myacq->cam_param[myacq->cam_idx].triggerout_delaytime,
	    nWidth = myacq->cam_param[myacq->cam_idx].triggerout_width;
	CameraSetTriggerOut(&nTgrOutPort, &nTgrOutMode, &nEdgeOutMode, &nDelayOutTm, &nWidth);

	/* exposure */
	CameraSetExposure((double)myacq->cam_param[myacq->cam_idx].exposure);

	/* gain */
	CameraSetGain((double)myacq->cam_param[myacq->cam_idx].gain);

	/* temperature */
	CameraSetTemperature((double)myacq->cam_param[myacq->cam_idx].temperature);

	INFO_STREAM << __FUNCTION__ << " camera initialized" << endl;	

	return TUCSENERR_SUCCESS;

}


/*
 * TUCSENERR acqthread::CameraBufferInit(void)
 * Initialize camera data buffer
 */
TUCSENERR acqthread::CameraBufferInit(void)
{
	int tucsen_err = TUCSENERR_SUCCESS;

	usleep(10000);

	INFO_STREAM << __FUNCTION__ << " initialize camera buffer" << endl;	

	/* initialize image buffer */
	frame.pBuffer = NULL;
	frame.ucFormatGet = TUFRM_FMT_USUAl; // Frame data format
	frame.uiRsdSize = 1; // Number of frames captured once (TUCCM_TRIGGER_STANDARD may be greater than 1)
	// allocate frame buffer
	if ((tucsen_err = TUCAM_Buf_Alloc(opCam.hIdxTUCam, &frame)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", cannot allocate image buffer, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	return tucsen_err;	

}


// Start camera acquisition
TUCSENERR acqthread::CameraStartAcquisition()
{
	int tucsen_err = TUCSENERR_SUCCESS;

	usleep(10000);

	// starting camera acquisition 
	INFO_STREAM << __FUNCTION__ << " start camera acquisition mode " << GetTUCSENString(TUCSEN_TRIG_MODE_map, tgrAttr.nTgrMode) << endl;
	if ((tucsen_err = TUCAM_Cap_Start(opCam.hIdxTUCam, tgrAttr.nTgrMode)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to start camera acquisition, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	return tucsen_err;
}


// Abort buffer data transfer and stop camera
TUCSENERR acqthread::CameraStopAcquisition()
{
	int tucsen_err = TUCSENERR_SUCCESS;

	usleep(10000);

	// abort buffer acquisition
	INFO_STREAM << __FUNCTION__ << " abort buffer acquisition" << endl;
	if ((tucsen_err = TUCAM_Buf_AbortWait(opCam.hIdxTUCam)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to abort camera acquisition, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	usleep(10000);

	// stop camera acquisition
	INFO_STREAM << __FUNCTION__ << " stop camera acquisition" << endl;
	if ((tucsen_err = TUCAM_Cap_Stop(opCam.hIdxTUCam)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to stop camera acquisition, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	INFO_STREAM << __FUNCTION__ << ", camera acquisition stopped" << endl;

	return tucsen_err;

}


/*
 * TUCSENERR acqthread::CameraReleaseAPI()
 * Release buffer, close camera connection and deinit the API env
 */
TUCSENERR acqthread::CameraReleaseAPI()
{
	int tucsen_err = TUCSENERR_SUCCESS;

	usleep(10000);

	// release the frame buffer
	INFO_STREAM << __FUNCTION__ << " release frame buffer" << endl;
	if ((tucsen_err = TUCAM_Buf_Release(opCam.hIdxTUCam)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to release buffer, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	usleep(10000);

	// close camera connection
	INFO_STREAM << __FUNCTION__ << " close camera connection" << endl;
	if ((tucsen_err = TUCAM_Dev_Close(opCam.hIdxTUCam)) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to release buffer, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	usleep(10000);

	// de-initialize SDK API environment
	INFO_STREAM << __FUNCTION__ << " de-initialize API" << endl;
	if ((tucsen_err = TUCAM_Api_Uninit()) != TUCAMRET_SUCCESS) {
		INFO_STREAM << __FUNCTION__ << ", failed to de-initialize API, " << 
			GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err) << endl;
		return tucsen_err;
	}

	return tucsen_err;

}


// update tango server attributes (max 1 sec. rate)
TUCSENERR acqthread::update_attributes(bool force_read)
{
	int tucsen_err = TUCSENERR_SUCCESS, tucsen_first_err = TUCSENERR_SUCCESS;
	int cnt_err = 0;
	struct timespec now;

	clock_gettime(CLOCK_REALTIME, &now);

	update_attribute_mutex.lock();
	
	// read temperature to keep the temperature regulation loop running
	if ((tucsen_err = CameraGetTemperature(false, NULL)) == TUCAMRET_SUCCESS) {
		*myacq->attr_Temperature_read = temperature;
	}
	else { 
		if (tucsen_first_err == TUCSENERR_SUCCESS)
			tucsen_first_err = tucsen_err;
	}


	if (((myacq->difftimems(data_refresh, now) < 1) || (*myacq->attr_EnableCameraReadings_read == false)) && (force_read == false)) {
		update_attribute_mutex.unlock();
		return tucsen_err;
	}

	if ((tucsen_err = CameraGetTriggerMode(force_read, NULL, NULL, NULL, NULL, NULL)) == TUCAMRET_SUCCESS) {
		*myacq->attr_TriggerMode_read = tgrAttr.nTgrMode;
		*myacq->attr_TriggerExpMode_read = tgrAttr.nExpMode;
		*myacq->attr_TriggerEdgeMode_read = tgrAttr.nEdgeMode;
		*myacq->attr_TriggerDelayTime_read = tgrAttr.nDelayTm;
		if (tgrAttr.nTgrMode == TUCCM_SEQUENCE) // free running (0)
			*myacq->attr_Trigger_read = false;
		else
			*myacq->attr_Trigger_read = true;
	}
	else {
		snprintf(*myacq->attr_TriggerInfo_read, TUCSENCAM_DESC, ", error ", GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err).c_str());
		if (tucsen_first_err == TUCSENERR_SUCCESS)
			tucsen_first_err = tucsen_err;

	}

	if ((tucsen_err = CameraGetTriggerOut(force_read, NULL, NULL, NULL, NULL, NULL)) == TUCAMRET_SUCCESS) {
		*myacq->attr_TriggerOutPort_read = tgrOutAttr.nTgrOutPort;
		*myacq->attr_TriggerOutMode_read = tgrOutAttr.nTgrOutMode;
		*myacq->attr_TriggerOutEdgeMode_read = tgrOutAttr.nEdgeMode;
		*myacq->attr_TriggerOutDelayTime_read = tgrOutAttr.nDelayTm;	
		*myacq->attr_TriggerOutWidth_read = tgrOutAttr.nWidth;
	}
	else {
		snprintf(*myacq->attr_TriggerOutInfo_read, TUCSENCAM_DESC, ", error ", GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err).c_str());
		if (tucsen_first_err == TUCSENERR_SUCCESS)
			tucsen_first_err = tucsen_err;
	}

	if ((tucsen_err = CameraGetExposure(force_read, NULL)) == TUCAMRET_SUCCESS) {
		*myacq->attr_Exposure_read = exposure;
	}
	else {
		if (tucsen_first_err == TUCSENERR_SUCCESS)
			tucsen_first_err = tucsen_err;
	}

	if ((tucsen_err = CameraGetGain(force_read, NULL)) == TUCAMRET_SUCCESS) {
		*myacq->attr_Gain_read = gain;
	}
	else { 
		if (tucsen_first_err == TUCSENERR_SUCCESS)
			tucsen_first_err = tucsen_err;
	}

	if (tucsen_first_err != TUCAMRET_SUCCESS) {
		snprintf(*myacq->attr_Model_read, TUCSENCAM_DESC, " error");
		snprintf(*myacq->attr_CameraInfo_read, TUCSENCAM_DESC, " error");	
		update_attribute_mutex.unlock();
		return tucsen_first_err;
	}

	*myacq->attr_ImageDepth_read = 16;
	*myacq->attr_TemperatureSetpoint_read = temperature_setpoint;
	snprintf(*myacq->attr_Model_read, TUCSENCAM_DESC, "%s", camera_model.c_str());
	snprintf(*myacq->attr_CameraInfo_read, TUCSENCAM_DESC, "%s", CameraGetInfo().c_str());
	snprintf(*myacq->attr_TriggerInfo_read, TUCSENCAM_DESC, "%s", CameraGetTriggerInfo().c_str());
	snprintf(*myacq->attr_TriggerOutInfo_read, TUCSENCAM_DESC, "%s", CameraGetTriggerOutInfo().c_str());

	clock_gettime(CLOCK_REALTIME, &data_refresh);

	read_counter++;

	update_attribute_mutex.unlock();

	return TUCAMRET_SUCCESS;	

}


// stop streaming
void acqthread::camera_stop()
{

	CameraStopAcquisition();

	if (myacq->run_flag)	
		myacq->abort_flag = true;

}


/* 
 *	acqthread::run()
 *	Run  
 */		
void acqthread::run(void *) 
{
	int32_t cnt_err = 0, cnt_err_old = 0, diff_frame,
		bn_err = 0, bn_err_old = 0, tucsen_err = TUCSENERR_SUCCESS;

	struct timespec time_old, time_new, tim_bn_err, tim_cnt_err, now;

	uint32_t camera_idx = 0;
	uint32_t wait_time_image = 0;
	uint32_t old_bunch_number = 0;
	uint32_t trig_err_consec_cnt = 0;
	uint32_t FrameCount = 0;

	unsigned char *pImg; // pointer to img data 

	struct sched_param mysched;

	//TUCSENERR tucsen_err = TUCSENERR_SUCCESS;

	myacq->run_flag = true;
	myacq->frequency = 10;
	myacq->bunch_number = 0;

	read_counter = 0;

	clock_gettime(CLOCK_REALTIME, &tim_cnt_err);
	clock_gettime(CLOCK_REALTIME, &tim_bn_err);
	tim_bn_err.tv_sec -= TUCSENCAM_ERR_TIME_BARRIER-1;
	tim_cnt_err.tv_sec -= TUCSENCAM_ERR_TIME_BARRIER-1;
	init_timeouts();

	// change thread priority...if possible
	sched_getparam(0, &mysched);
	mysched.sched_priority = sched_get_priority_max(SCHED_RR)/2 - 1;
	int sched_err;
	if( (sched_err=sched_setscheduler(0, SCHED_RR, &mysched )) < 0) {
		cout << sched_err << endl;
	}
	// omniorb priority
	set_priority(PRIORITY_HIGH);

	myacq->set_state(Tango::INIT);
	myacq->set_status("Initializing camera");


	//openCamera();
	tucsen_err = CameraInitAPI();

	//initCamera();
	tucsen_err = CameraInit();

	tucsen_err = CameraBufferInit();

	init_data();

	myacq->init_buffers(myacq);

	set_roi();

	tucsen_err = CameraStartAcquisition();

	int counter = 0;
	while (!myacq->abort_flag) {

		if (myacq->pause_flag == true) {
			myacq->set_status("acquisition paused");
			usleep(10000);
			continue; // go to while
		}

		/*if(tgrAttr.nTgrMode == TUCCM_TRIGGER_SOFTWARE){
			tucsen_err = TUCAM_Cap_DoSoftwareTrigger(opCam.hIdxTUCam);
		}*/

		tucsen_err = TUCAMRET_SUCCESS;
		frame.ucFormatGet = TUFRM_FMT_USUAl;		
        	tucsen_err = TUCAM_Buf_WaitForFrame(opCam.hIdxTUCam,&frame);

		tucsen_err = TUCAM_Buf_CopyFrame(opCam.hIdxTUCam,&frame);

		if(tucsen_err != TUCAMRET_SUCCESS) {

			myacq->set_state(Tango::OFF);
			if (myacq->cam_param[myacq->cam_idx].trigger) 
				myacq->set_status("acquisition failed (trigger?)");
			usleep(200000);
			if ((trig_err_consec_cnt++ > TUCSENCAM_TRIG_ERR_MAX_CNT) && (myacq->autoReconnect == false)) {
				myacq->abort_flag = true;
				trig_err_consec_cnt = 0;
				myacq->set_state(Tango::FAULT);
				myacq->set_status("reached maximum number of autorestart, acquisition stopped");
			}

		}

		else {

			//printf("GG1 *myacq->attr_TriggerEdgeMode_read = %d\n",*myacq->attr_TriggerEdgeMode_read);

			clock_gettime(CLOCK_REALTIME, &bunch_number_dataready_time);

			*myacq->attr_BunchNumberDataReadyDelay_read = myacq->difftimems(myacq->bunch_number_switch_time,bunch_number_dataready_time);

			//printf("GG2 *myacq->attr_TriggerEdgeMode_read = %d\n",*myacq->attr_TriggerEdgeMode_read);

#ifdef TUCSENCAM_RNM
			if (myacq->rnmEnabled) {
				char check_flag;						
				rnm_bunch_number_get_value(0, 1, &myacq->bunch_number, &check_flag);

				/* vcheck bunch number lost */
				if ((old_bunch_number > 0) && ((myacq->bunch_number - old_bunch_number) != 1)) {
					bn_err++;
					myacq->error_counters[TUCSENCAM_ERR_BN_LOST]++;
				}
				old_bunch_number = myacq->bunch_number;
				counter = myacq->bunch_number;
				

				struct timeval tv;
				gettimeofday(&tv, NULL);
				struct tm gmt;
				gmtime_r(&tv.tv_sec, &gmt);
				uint64_t gmt_seconds = (uint64_t)mktime(&gmt);
				uint64_t gmt_microseconds = (uint64_t)tv.tv_usec;

				//printf("get data bunch number = %d %lld %lld\n",myacq->bunch_number,gmt_seconds,gmt_microseconds);
			}
			else {
				counter++;
				myacq->bunch_number = counter;
			}
#else
			counter++;
			myacq->bunch_number = FrameCount;
			
#endif

			//printf("GG3 *myacq->attr_TriggerEdgeMode_read = %d\n",*myacq->attr_TriggerEdgeMode_read);

			myacq->bunch_number += *myacq->attr_BunchNumberOffset_read;

			diff_frame = frame.uiIndex+1;
			if (diff_frame > 1) {
				myacq->error_counters[TUCSENCAM_ERR_CNT_LOST]++;
				cnt_err++;
			}

			// calculate frame rate
			clock_gettime(CLOCK_REALTIME, &time_new);
			double curr_frequency = (double)diff_frame/myacq->difftimems(time_old,time_new);
			if ((curr_frequency > 0) && (curr_frequency < 700)) {
				myacq->frequency = curr_frequency*0.1+myacq->frequency*0.9; /* low pass filter */
			}
				time_old.tv_sec = time_new.tv_sec; time_old.tv_nsec = time_new.tv_nsec;

			//int elemByteSize = frame.ucElemBytes; // size of image element

			pImg = frame.pBuffer + frame.usHeader; // pointer to actual img data

			// update frame size
			width = frame.usWidth;
			height = frame.usHeight;

			//printf("GG4 *myacq->attr_TriggerEdgeMode_read = %d\n",*myacq->attr_TriggerEdgeMode_read);
	
			/*
			if (counter > 1) {
				myacq->error_counters[TUCSENCAM_ERR_CNT_LOST]++;
				cnt_err++;
			}*/

			// resize data structure, if necessary
			update_configuration();

			// store data into circular buffer and perform the processing
			save_data(pImg,counter);

			myacq->set_state(Tango::ON);
			myacq->set_status("acquisition running");
				
			tucsen_err = update_attributes(false);

			if (tucsen_err != TUCSENERR_SUCCESS) {
				myacq->set_state(Tango::FAULT);
				myacq->set_status(GetTUCSENString(TUCSEN_ERR_MSG_map, tucsen_err).c_str());
				myacq->abort_flag = true;
			}
			
			/* notify frame lost error */
			stringstream err_msg_cnt, err_msg_bn;
			if (abs(cnt_err_old-cnt_err) > 0) 
				clock_gettime(CLOCK_REALTIME, &tim_cnt_err);
			if (myacq->difftimems(tim_cnt_err,now) < TUCSENCAM_ERR_TIME_BARRIER)  
				err_msg_cnt << "missing frames error  ";

			/* notify bunch number lost error */
			if (abs(bn_err_old-bn_err) > 0) 
				clock_gettime(CLOCK_REALTIME, &tim_bn_err);
			if (myacq->difftimems(tim_bn_err,now) < TUCSENCAM_ERR_TIME_BARRIER)  
				err_msg_bn << "bunch number error";

			/* store old error counters */
			cnt_err_old = cnt_err;bn_err_old = bn_err;
			clock_gettime(CLOCK_REALTIME, &now);

			/* make error message persistent for a few seconds */
			if ((myacq->difftimems(tim_cnt_err,now) < TUCSENCAM_ERR_TIME_BARRIER) ||
					(myacq->difftimems(tim_bn_err,now) < TUCSENCAM_ERR_TIME_BARRIER)
			    && (myacq->get_state() == (Tango::ON))) {
				stringstream err_msg;
				err_msg << err_msg_cnt.str() << err_msg_bn.str();
				myacq->set_status(err_msg.str());
			}

			//printf("GG5 *myacq->attr_TriggerEdgeMode_read = %d\n",*myacq->attr_TriggerEdgeMode_read);

		}


	} /* while runflag */

	CameraStopAcquisition();

	CameraReleaseAPI();

	delete_data();

	myacq->delete_buffers(myacq);

	malloc_trim(0); /* release heap */
	
	myacq->pause_flag = false;
	myacq->run_flag = false;
	myacq->abort_flag = false;

	if (myacq->get_state() != Tango::FAULT) {
		myacq->set_state(Tango::OFF); 
		myacq->set_status("acquisition not running");
	}

}
